@@ -2,28 +2,29 @@
  * device.h: DVD device access
  *****************************************************************************
  * Copyright (C) 1998-2006 VideoLAN
+ * $Id$
  *
  * Authors: Stéphane Borel <stef@via.ecp.fr>
  *          Sam Hocevar <sam@zoy.org>
  *          Håkan Hjort <d95hjort@dtek.chalmers.se>
  *
- * This library is free software; you can redistribute it and/or modify
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this library; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  *****************************************************************************/
 
 /*
-	Modifications for XBMC are all contained within WITH_CACHE
+	Modifications for XBMC are all contained within _XBOX (real xbox hardware) or WITH_CACHE
 */
 
 /*****************************************************************************
@@ -52,6 +53,12 @@
 #   include <limits.h>
 #endif
 
+#if defined( WIN32 ) && !defined( SYS_CYGWIN )
+#   include <io.h>                                                 /* read() */
+#else
+#   include <sys/uio.h>                                      /* struct iovec */
+#endif
+
 #ifdef DARWIN_DVD_IOCTL
 #   include <paths.h>
 #   include <CoreFoundation/CoreFoundation.h>
@@ -62,14 +69,6 @@
 #   include <IOKit/storage/IODVDMedia.h>
 #endif
 
-#ifdef __OS2__
-#   define INCL_DOS
-#   define INCL_DOSDEVIOCTL
-#   include <os2.h>
-#   include <io.h>                                              /* setmode() */
-#   include <fcntl.h>                                           /* O_BINARY  */
-#endif
-
 #include "dvdcss/dvdcss.h"
 
 #include "common.h"
@@ -96,12 +95,6 @@ static int aspi_read  ( dvdcss_t, void *, int );
 static int win_readv  ( dvdcss_t, struct iovec *, int );
 
 static int aspi_read_internal  ( int, void *, int );
-#elif defined( __OS2__ )
-static int os2_open ( dvdcss_t, char const * );
-/* just use macros for libc */
-#   define os2_seek     libc_seek
-#   define os2_read     libc_read
-#   define os2_readv    libc_readv
 #endif
 
 int _dvdcss_use_ioctls( dvdcss_t dvdcss )
@@ -121,16 +114,6 @@ int _dvdcss_use_ioctls( dvdcss_t dvdcss )
     {
         return 1;
     }
-#elif defined( __OS2__ )
-    ULONG ulMode;
-
-    if( DosQueryFHState( dvdcss->i_fd, &ulMode ) != 0 )
-        return 1;  /* What to do?  Be conservative and try to use the ioctls */
-
-    if( ulMode & OPEN_FLAGS_DASD )
-        return 1;
-
-    return 0;
 #else
     struct stat fileinfo;
     int ret;
@@ -151,6 +134,7 @@ int _dvdcss_use_ioctls( dvdcss_t dvdcss )
      * OpenBSD  needs a char device
      * NetBSD   needs a char device
      * FreeBSD  can use either the block or the char device
+     * BSD/OS   can use either the block or the char device
      */
 
     /* Check if this is a block/char device */
@@ -177,28 +161,6 @@ void _dvdcss_check ( dvdcss_t dvdcss )
     kern_return_t kern_result;
     io_iterator_t media_iterator;
     CFMutableDictionaryRef classes_to_match;
-#elif defined( __OS2__ )
-#pragma pack( 1 )
-    struct
-    {
-        BYTE bCmdInfo;
-        BYTE bDrive;
-    } param;
-
-    struct
-    {
-        BYTE    abEBPB[31];
-        USHORT  usCylinders;
-        BYTE    bDevType;
-        USHORT  usDevAttr;
-    } data;
-#pragma pack()
-
-    ULONG ulParamLen;
-    ULONG ulDataLen;
-    ULONG rc;
-
-    int i;
 #else
     char *ppsz_devices[] = { "/dev/dvd", "/dev/cdrom", "/dev/hdc", NULL };
     int i, i_fd;
@@ -235,7 +197,7 @@ void _dvdcss_check ( dvdcss_t dvdcss )
         /* Remove trailing backslash */
         psz_device[2] = '\0';
 
-        /* FIXME: we want to differentiate between CD and DVD drives
+        /* FIXME: we want to differenciate between CD and DVD drives
          * using DeviceIoControl() */
         print_debug( dvdcss, "defaulting to drive `%s'", psz_device );
         free( dvdcss->psz_device );
@@ -312,32 +274,6 @@ void _dvdcss_check ( dvdcss_t dvdcss )
     }
 
     IOObjectRelease( media_iterator );
-#elif defined( __OS2__ )
-    for( i = 0; i < 26; i++ )
-    {
-        param.bCmdInfo = 0;
-        param.bDrive = i;
-
-        rc = DosDevIOCtl( ( HFILE )-1, IOCTL_DISK, DSK_GETDEVICEPARAMS,
-                          &param, sizeof( param ), &ulParamLen,
-                          &data, sizeof( data ), &ulDataLen );
-
-        if( rc == 0 )
-        {
-            /* Check for removable and for cylinders */
-            if( ( data.usDevAttr & 1 ) == 0 && data.usCylinders == 0xFFFF )
-            {
-                char psz_dvd[] = "A:";
-
-                psz_dvd[0] += i;
-
-                print_debug( dvdcss, "defaulting to drive `%s'", psz_dvd );
-                free( dvdcss->psz_device );
-                dvdcss->psz_device = strdup( psz_dvd );
-                return;
-            }
-        }
-    }
 #else
     for( i = 0; ppsz_devices[i]; i++ )
     {
@@ -364,11 +300,20 @@ int _dvdcss_open ( dvdcss_t dvdcss )
 
 #if defined( WIN32 )
     dvdcss->b_file = 1;
+#if defined( _XBOX )
+	// If we've passed over the device string make sure we don't try
+	// to use file based handling (libc) - we want Win2k routines ...
+	if (!stricmp(psz_device, "\\Device\\Cdrom0"))
+		dvdcss->b_file = 0;
+	else
+		dvdcss->b_file = stricmp(psz_device, "D:");
+#else
     /* If device is "X:" or "X:\", we are not actually opening a file. */
     if (psz_device[0] && psz_device[1] == ':' &&
        (!psz_device[2] || (psz_device[2] == '\\' && !psz_device[3])))
         dvdcss->b_file = 0;
 
+#endif	// _XBOX
     /* Initialize readv temporary buffer */
     dvdcss->p_readv_buffer   = NULL;
     dvdcss->i_readv_buf_size = 0;
@@ -390,18 +335,6 @@ int _dvdcss_open ( dvdcss_t dvdcss )
         return aspi_open( dvdcss, psz_device );
     }
     else
-#elif defined( __OS2__ )
-    /* If device is "X:" or "X:\", we are not actually opening a file. */
-    if( psz_device[0] && psz_device[1] == ':' &&
-        ( !psz_device[2] || ( psz_device[2] == '\\' && !psz_device[3] ) ) )
-    {
-        print_debug( dvdcss, "using OS2 API for access" );
-        dvdcss->pf_seek  = os2_seek;
-        dvdcss->pf_read  = os2_read;
-        dvdcss->pf_readv = os2_readv;
-        return os2_open( dvdcss, psz_device );
-    }
-    else
 #endif
     {
         print_debug( dvdcss, "using libc for access" );
@@ -412,7 +345,7 @@ int _dvdcss_open ( dvdcss_t dvdcss )
     }
 }
 
-#ifdef DVDCSS_RAW_OPEN
+#ifndef WIN32
 int _dvdcss_raw_open ( dvdcss_t dvdcss, char const *psz_device )
 {
     dvdcss->i_raw_fd = open( psz_device, 0 );
@@ -446,11 +379,13 @@ int _dvdcss_close ( dvdcss_t dvdcss )
     }
     else /* ASPI */
     {
+#if !defined(_XBOX)
         struct w32_aspidev *fd = (struct w32_aspidev *) dvdcss->i_fd;
 
-        /* Unload ASPI and free w32_aspidev structure */
+        /* Unload aspi and free w32_aspidev structure */
         FreeLibrary( (HMODULE) fd->hASPI );
         free( (void*) dvdcss->i_fd );
+#endif	// !_XBOX
     }
 
     /* Free readv temporary buffer */
@@ -465,13 +400,11 @@ int _dvdcss_close ( dvdcss_t dvdcss )
 #else
     close( dvdcss->i_fd );
 
-#ifdef DVDCSS_RAW_OPEN
     if( dvdcss->i_raw_fd >= 0 )
     {
         close( dvdcss->i_raw_fd );
         dvdcss->i_raw_fd = -1;
     }
-#endif
 
     return 0;
 #endif
@@ -484,7 +417,11 @@ int _dvdcss_close ( dvdcss_t dvdcss )
  *****************************************************************************/
 static int libc_open ( dvdcss_t dvdcss, char const *psz_device )
 {
+#if !defined( WIN32 )
+    dvdcss->i_fd = dvdcss->i_read_fd = open( psz_device, 0 );
+#else
     dvdcss->i_fd = dvdcss->i_read_fd = open( psz_device, O_BINARY );
+#endif
 
     if( dvdcss->i_fd == -1 )
     {
@@ -502,13 +439,18 @@ static int libc_open ( dvdcss_t dvdcss, char const *psz_device )
 #if defined( WIN32 )
 static int win2k_open ( dvdcss_t dvdcss, char const *psz_device )
 {
+#ifdef _XBOX
+    char psz_dvd[70];
+    strcpy(psz_dvd, "cdrom0:");
+#else
     char psz_dvd[7];
-    snprintf( psz_dvd, 7, "\\\\.\\%c:", psz_device[0] );
+    _snprintf( psz_dvd, 7, "\\\\.\\%c:", psz_device[0] );
 
+#endif
     /* To work around an M$ bug in IOCTL_DVD_READ_STRUCTURE, we need read
      * _and_ write access to the device (so we can make SCSI Pass Through
      * Requests). Unfortunately this is only allowed if you have
-     * administrator privileges so we allow for a fallback method with
+     * administrator priviledges so we allow for a fallback method with
      * only read access to the device (in this case ioctl_ReadCopyright()
      * won't send back the right result).
      * (See Microsoft Q241374: Read and Write Access Required for SCSI
@@ -518,7 +460,7 @@ static int win2k_open ( dvdcss_t dvdcss, char const *psz_device )
     DWORD flags = FILE_FLAG_NO_BUFFERING; /* we handle buffering ourself */
 #else
     DWORD flags = FILE_FLAG_RANDOM_ACCESS;
-#endif //!WITH_CACHE
+#endif //!_XBOX
 
     dvdcss->i_fd = (int)
                 CreateFile( psz_dvd, GENERIC_READ | GENERIC_WRITE,
@@ -553,7 +495,7 @@ static int aspi_open( dvdcss_t dvdcss, char const * psz_device )
     SENDASPI32COMMAND lpSendCommand;
     char c_drive = psz_device[0];
 
-    /* load ASPI and init w32_aspidev structure */
+    /* load aspi and init w32_aspidev structure */
     hASPI = LoadLibrary( "wnaspi32.dll" );
     if( hASPI == NULL )
     {
@@ -566,7 +508,7 @@ static int aspi_open( dvdcss_t dvdcss, char const * psz_device )
 
     if(lpGetSupport == NULL || lpSendCommand == NULL )
     {
-        print_error( dvdcss, "unable to get ASPI function pointers" );
+        print_error( dvdcss, "unable to get aspi function pointers" );
         FreeLibrary( hASPI );
         return -1;
     }
@@ -582,7 +524,7 @@ static int aspi_open( dvdcss_t dvdcss, char const * psz_device )
 
     if( HIBYTE( LOWORD ( dwSupportInfo ) ) != SS_COMP )
     {
-        print_error( dvdcss, "unable to initialize ASPI layer" );
+        print_error( dvdcss, "unable to initalize aspi layer" );
         FreeLibrary( hASPI );
         return -1;
     }
@@ -627,9 +569,10 @@ static int aspi_open( dvdcss_t dvdcss, char const * psz_device )
             if( (srbDiskInfo.SRB_Status == SS_COMP) &&
                 (srbDiskInfo.SRB_Int13HDriveInfo == c_drive) )
             {
-                /* Make sure this is a CD-ROM device */
-                struct SRB_GDEVBlock srbGDEVBlock = { 0 };
+                /* Make sure this is a cdrom device */
+                struct SRB_GDEVBlock srbGDEVBlock;
 
+                memset( &srbGDEVBlock, 0, sizeof(struct SRB_GDEVBlock) );
                 srbGDEVBlock.SRB_Cmd    = SC_GET_DEV_TYPE;
                 srbGDEVBlock.SRB_HaId   = i;
                 srbGDEVBlock.SRB_Target = j;
@@ -648,7 +591,7 @@ static int aspi_open( dvdcss_t dvdcss, char const * psz_device )
                 {
                     free( (void*) fd );
                     FreeLibrary( hASPI );
-                    print_error( dvdcss,"this is not a CD-ROM drive" );
+                    print_error( dvdcss,"this is not a cdrom drive" );
                     return -1;
                 }
             }
@@ -657,42 +600,11 @@ static int aspi_open( dvdcss_t dvdcss, char const * psz_device )
 
     free( (void*) fd );
     FreeLibrary( hASPI );
-    print_error( dvdcss, "unable to get haid and target (ASPI)" );
+    print_error( dvdcss, "unable to get haid and target (aspi)" );
     return -1;
 }
 #endif
 
-#ifdef __OS2__
-static int os2_open ( dvdcss_t dvdcss, char const *psz_device )
-{
-    char  psz_dvd[] = "X:";
-    HFILE hfile;
-    ULONG ulAction;
-    ULONG rc;
-
-    psz_dvd[0] = psz_device[0];
-
-    rc = DosOpenL( ( PSZ )psz_dvd, &hfile, &ulAction, 0, FILE_NORMAL,
-                   OPEN_ACTION_OPEN_IF_EXISTS | OPEN_ACTION_FAIL_IF_NEW,
-                   OPEN_ACCESS_READONLY | OPEN_SHARE_DENYNONE | OPEN_FLAGS_DASD,
-                   NULL );
-
-    if( rc )
-    {
-        print_error( dvdcss, "failed to open device" );
-        return -1;
-    }
-
-    setmode( hfile, O_BINARY );
-
-    dvdcss->i_fd = dvdcss->i_read_fd = hfile;
-
-    dvdcss->i_pos = 0;
-
-    return 0;
-}
-#endif
-
 /*****************************************************************************
  * Seek commands.
  *****************************************************************************/
@@ -920,7 +832,7 @@ static int win2k_read ( dvdcss_t dvdcss, void *p_buffer, int i_blocks )
 
     dvdcss->i_pos += i_bytes / DVDCSS_BLOCK_SIZE;
     return i_bytes / DVDCSS_BLOCK_SIZE;
-#endif // WITH_CACHE	
+#endif // WITH_CACHE
 }
 
 static int aspi_read ( dvdcss_t dvdcss, void *p_buffer, int i_blocks )
@@ -965,8 +877,8 @@ static int libc_readv ( dvdcss_t dvdcss, struct iovec *p_iovec, int i_blocks )
         if( i_bytes < 0 )
         {
             /* One of the reads failed, too bad.
-             * We won't even bother returning the reads that went OK,
-             * and as in the POSIX spec the file position is left
+             * We won't even bother returning the reads that went ok,
+             * and as in the posix spec the file postition is left
              * unspecified after a failure */
             dvdcss->i_pos = -1;
             return -1;
@@ -1052,7 +964,7 @@ static int win_readv ( dvdcss_t dvdcss, struct iovec *p_iovec, int i_blocks )
                        i_blocks_total * DVDCSS_BLOCK_SIZE, &i_bytes, NULL ) )
         {
             /* The read failed... too bad.
-             * As in the POSIX spec the file position is left
+             * As in the posix spec the file postition is left
              * unspecified after a failure */
             dvdcss->i_pos = -1;
             return -1;
@@ -1094,7 +1006,7 @@ static int win_readv ( dvdcss_t dvdcss, struct iovec *p_iovec, int i_blocks )
 static int aspi_read_internal( int i_fd, void *p_data, int i_blocks )
 {
     HANDLE hEvent;
-    struct SRB_ExecSCSICmd ssc = { 0 };
+    struct SRB_ExecSCSICmd ssc;
     struct w32_aspidev *fd = (struct w32_aspidev *) i_fd;
 
     /* Create the transfer completion event */
@@ -1104,6 +1016,8 @@ static int aspi_read_internal( int i_fd, void *p_data, int i_blocks )
         return -1;
     }
 
+    memset( &ssc, 0, sizeof( ssc ) );
+
     ssc.SRB_Cmd         = SC_EXEC_SCSI_CMD;
     ssc.SRB_Flags       = SRB_DIR_IN | SRB_EVENT_NOTIFY;
     ssc.SRB_HaId        = LOBYTE( fd->i_sid );
@@ -1120,7 +1034,7 @@ static int aspi_read_internal( int i_fd, void *p_data, int i_blocks )
     ssc.CDBByte[4]      = (UCHAR) (fd->i_blocks >> 8) & 0xff;
     ssc.CDBByte[5]      = (UCHAR) (fd->i_blocks) & 0xff;
 
-    /* We have to break down the reads into 64KB pieces (ASPI restriction) */
+    /* We have to break down the reads into 64kb pieces (ASPI restriction) */
     if( i_blocks > 32 )
     {
         ssc.SRB_BufLen = 32 * DVDCSS_BLOCK_SIZE;
@@ -1131,7 +1045,7 @@ static int aspi_read_internal( int i_fd, void *p_data, int i_blocks )
         ResetEvent( hEvent );
         fd->lpSendCommand( (void*) &ssc );
 
-        /* transfer the next 64KB (aspi_read_internal is called recursively)
+        /* transfer the next 64kb (aspi_read_internal is called recursively)
          * We need to check the status of the read on return */
         if( aspi_read_internal( i_fd,
                                 (uint8_t*) p_data + 32 * DVDCSS_BLOCK_SIZE,
