@@ -1,6 +1,6 @@
 /*
- *      Copyright (C) 2011-2013 Team XBMC
- *      http://xbmc.org
+ *      Copyright (C) 2011-2012 Team XBMC
+ *      http://www.xbmc.org
  *
  *  This Program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,9 +27,10 @@
 #include "guilib/LocalizeStrings.h"
 #include "filesystem/Directory.h"
 #include "settings/AdvancedSettings.h"
+#include "settings/GUISettings.h"
 #include "settings/Settings.h"
 #include "ApplicationMessenger.h"
-#include "filesystem/FavouritesDirectory.h"
+#include "Favourites.h"
 #include "utils/JobManager.h"
 #include "dialogs/GUIDialogYesNo.h"
 #include "addons/AddonManager.h"
@@ -80,16 +81,15 @@ void CAddonInstaller::OnJobComplete(unsigned int jobID, bool success, CJob* job)
   { // repo job finished
     m_repoUpdateDone.Set();
     m_repoUpdateJob = 0;
-    lock.Leave();
   }
   else
   { // download job
     JobMap::iterator i = find_if(m_downloadJobs.begin(), m_downloadJobs.end(), bind2nd(find_map(), jobID));
     if (i != m_downloadJobs.end())
       m_downloadJobs.erase(i);
-    lock.Leave();
     PrunePackageCache();
   }
+  lock.Leave();
 
   CGUIMessage msg(GUI_MSG_NOTIFY_ALL, 0, 0, GUI_MSG_UPDATE);
   g_windowManager.SendThreadMessage(msg);
@@ -112,7 +112,7 @@ void CAddonInstaller::OnJobProgress(unsigned int jobID, unsigned int progress, u
 bool CAddonInstaller::IsDownloading() const
 {
   CSingleLock lock(m_critSection);
-  return !m_downloadJobs.empty();
+  return m_downloadJobs.size() > 0;
 }
 
 void CAddonInstaller::GetInstallList(VECADDONS &addons) const
@@ -241,6 +241,9 @@ bool CAddonInstaller::DoInstall(const AddonPtr &addon, const CStdString &hash, b
     return false;
 
   // check whether all the dependencies are available or not
+  // TODO: we currently assume that dependencies will install correctly (and each of their dependencies and so on).
+  //       it may be better to install the dependencies first to minimise the chance of an addon becoming orphaned due to
+  //       missing deps.
   if (!CheckDependencies(addon))
   {
     CGUIDialogKaiToast::QueueNotification(addon->Icon(), addon->Name(), g_localizeStrings.Get(24044), TOAST_DISPLAY_TIME, false);
@@ -289,7 +292,7 @@ bool CAddonInstaller::InstallFromZip(const CStdString &path)
   if (xml.LoadFile(archive) && CAddonMgr::Get().LoadAddonDescriptionFromMemory(xml.RootElement(), addon))
   {
     // set the correct path
-    addon->Props().path = items[0]->GetPath();
+    addon->Props().path = path;
 
     // install the addon
     return DoInstall(addon);
@@ -308,22 +311,12 @@ void CAddonInstaller::InstallFromXBMCRepo(const set<CStdString> &addonIDs)
     Install(*i);
 }
 
-bool CAddonInstaller::CheckDependencies(const AddonPtr &addon, CAddonDatabase *database /* = NULL */)
-{
-  std::vector<std::string> preDeps;
-  preDeps.push_back(addon->ID());
-  CAddonDatabase localDB;
-  if (!database)
-    database = &localDB;
-  return CheckDependencies(addon, preDeps, *database);
-}
-
-bool CAddonInstaller::CheckDependencies(const AddonPtr &addon,
-                                        std::vector<std::string>& preDeps, CAddonDatabase &database)
+bool CAddonInstaller::CheckDependencies(const AddonPtr &addon)
 {
   if (!addon.get())
     return true; // a NULL addon has no dependencies
   ADDONDEPS deps = addon->GetDeps();
+  CAddonDatabase database;
   database.Open();
   for (ADDONDEPS::const_iterator i = deps.begin(); i != deps.end(); ++i)
   {
@@ -336,43 +329,24 @@ bool CAddonInstaller::CheckDependencies(const AddonPtr &addon,
     { // we have it but our version isn't good enough, or we don't have it and we need it
       if (!database.GetAddon(addonID, dep) || !dep->MeetsVersion(version))
       { // we don't have it in a repo, or we have it but the version isn't good enough, so dep isn't satisfied.
-        CLog::Log(LOGDEBUG, "Addon %s requires %s version %s which is not available", addon->ID().c_str(), addonID.c_str(), version.asString().c_str());
-        database.Close();
+        CLog::Log(LOGDEBUG, "Addon %s requires %s version %s which is not available", addon->ID().c_str(), addonID.c_str(), version.c_str());
         return false;
       }
     }
+    // prevent infinite loops
+    if (dep && dep->ID() == addon->ID())
+    {
+      CLog::Log(LOGERROR, "Addon %s depends on itself, ignoring", addon->ID().c_str());
+      return false;
+    }
     // at this point we have our dep, or the dep is optional (and we don't have it) so check that it's OK as well
     // TODO: should we assume that installed deps are OK?
-    if (dep && std::find(preDeps.begin(), preDeps.end(), dep->ID()) == preDeps.end())
-    {
-      if (!CheckDependencies(dep, preDeps, database))
-      {
-        database.Close();
-        return false;
-      }
-      preDeps.push_back(dep->ID());
-    }
+    if (dep && !CheckDependencies(dep))
+      return false;
   }
-  database.Close();
   return true;
 }
 
-CDateTime CAddonInstaller::LastRepoUpdate() const
-{
-  CDateTime update;
-  VECADDONS addons;
-  CAddonMgr::Get().GetAddons(ADDON_REPOSITORY,addons);
-  for (unsigned int i=0;i<addons.size();++i)
-  {
-    CAddonDatabase database;
-    database.Open();
-    CDateTime lastUpdate = database.GetRepoTimestamp(addons[i]->ID());
-    if (lastUpdate.IsValid() && lastUpdate > update)
-      update = lastUpdate;
-  }
-  return update;
-}
-
 void CAddonInstaller::UpdateRepos(bool force, bool wait)
 {
   CSingleLock lock(m_critSection);
@@ -399,7 +373,7 @@ void CAddonInstaller::UpdateRepos(bool force, bool wait)
     CAddonDatabase database;
     database.Open();
     CDateTime lastUpdate = database.GetRepoTimestamp(addons[i]->ID());
-    if (force || !lastUpdate.IsValid() || lastUpdate + CDateTimeSpan(0,24,0,0) < CDateTime::GetCurrentDateTime())
+    if (force || !lastUpdate.IsValid() || lastUpdate + CDateTimeSpan(0,6,0,0) < CDateTime::GetCurrentDateTime())
     {
       CLog::Log(LOGDEBUG,"Checking repositories for updates (triggered by %s)",addons[i]->Name().c_str());
       m_repoUpdateJob = CJobManager::GetInstance().AddJob(new CRepositoryUpdateJob(addons), this);
@@ -424,28 +398,25 @@ void CAddonInstaller::PrunePackageCache()
 {
   std::map<CStdString,CFileItemList*> packs;
   int64_t size = EnumeratePackageFolder(packs);
-  int64_t limit = (int64_t)g_advancedSettings.m_addonPackageFolderSize*1024*1024;
+  int64_t limit = g_advancedSettings.m_addonPackageFolderSize*1024*1024;
   if (size < limit)
     return;
 
   // Prune packages
   // 1. Remove the largest packages, leaving at least 2 for each add-on
-  CFileItemList   items;
-  CAddonDatabase  db;
-  db.Open();
+  CFileItemList items;
   for (std::map<CStdString,CFileItemList*>::const_iterator it  = packs.begin();
                                                           it != packs.end();++it)
   {
-    it->second->Sort(SortByLabel, SortOrderDescending);
+    it->second->Sort(SORT_METHOD_LABEL,SortOrderDescending);
     for (int j=2;j<it->second->Size();++j)
       items.Add(CFileItemPtr(new CFileItem(*it->second->Get(j))));
   }
-  items.Sort(SortBySize, SortOrderDescending);
+  items.Sort(SORT_METHOD_SIZE,SortOrderDescending);
   int i=0;
   while (size > limit && i < items.Size())
   {
     size -= items[i]->m_dwSize;
-    db.RemovePackage(items[i]->GetPath());
     CFileUtils::DeleteItem(items[i++],true);
   }
 
@@ -459,12 +430,11 @@ void CAddonInstaller::PrunePackageCache()
       if (it->second->Size() > 1)
         items.Add(CFileItemPtr(new CFileItem(*it->second->Get(1))));
     }
-    items.Sort(SortByDate, SortOrderAscending);
+    items.Sort(SORT_METHOD_DATE,SortOrderAscending);
     i=0;
     while (size > limit && i < items.Size())
     {
       size -= items[i]->m_dwSize;
-      db.RemovePackage(items[i]->GetPath());
       CFileUtils::DeleteItem(items[i++],true);
     }
   }
@@ -499,106 +469,64 @@ CAddonInstallJob::CAddonInstallJob(const AddonPtr &addon, const CStdString &hash
 {
 }
 
-AddonPtr CAddonInstallJob::GetRepoForAddon(const AddonPtr& addon)
-{
-  CAddonDatabase database;
-  database.Open();
-  CStdString repo;
-  database.GetRepoForAddon(addon->ID(), repo);
-  AddonPtr repoPtr;
-  CAddonMgr::Get().GetAddon(repo, repoPtr);
-
-  return repoPtr;
-}
-
 bool CAddonInstallJob::DoWork()
 {
-  AddonPtr repoPtr = GetRepoForAddon(m_addon);
+  // Addons are installed by downloading the .zip package on the server to the local
+  // packages folder, then extracting from the local .zip package into the addons folder
+  // Both these functions are achieved by "copying" using the vfs.
+
+  CStdString dest="special://home/addons/packages/";
+  CStdString package = URIUtils::AddFileToFolder("special://home/addons/packages/",
+                                              URIUtils::GetFileName(m_addon->Path()));
+
   CStdString installFrom;
-  if (!repoPtr || repoPtr->Props().libname.empty())
+  if (URIUtils::HasSlashAtEnd(m_addon->Path()))
+  { // passed in a folder - all we need do is copy it across
+    installFrom = m_addon->Path();
+  }
+  else
   {
-    // Addons are installed by downloading the .zip package on the server to the local
-    // packages folder, then extracting from the local .zip package into the addons folder
-    // Both these functions are achieved by "copying" using the vfs.
-
-    CStdString dest="special://home/addons/packages/";
-    CStdString package = URIUtils::AddFileToFolder("special://home/addons/packages/",
-                                                URIUtils::GetFileName(m_addon->Path()));
-    if (URIUtils::HasSlashAtEnd(m_addon->Path()))
-    { // passed in a folder - all we need do is copy it across
-      installFrom = m_addon->Path();
-    }
-    else
+    // zip passed in - download + extract
+    CStdString path(m_addon->Path());
+    if (!m_referer.IsEmpty() && URIUtils::IsInternetStream(path))
     {
-      CStdString      md5;
-      CAddonDatabase  db;
-      db.Open();
-
-      // check that we don't already have a valid copy
-      if (!m_hash.empty() && CFile::Exists(package))
-      {
-        if (db.GetPackageHash(m_addon->ID(), package, md5) && m_hash != md5)
-        {
-          db.RemovePackage(package);
-          CFile::Delete(package);
-        }
-      }
-
-      // zip passed in - download + extract
-      if (!CFile::Exists(package))
-      {
-        CStdString path(m_addon->Path());
-        if (!m_referer.empty() && URIUtils::IsInternetStream(path))
-        {
-          CURL url(path);
-          url.SetProtocolOptions(m_referer);
-          path = url.Get();
-        }
-
-        if (!DownloadPackage(path, dest))
-        {
-          CFile::Delete(package);
-          return false;
-        }
-      }
-
-      // at this point we have the package - check that it is valid
-      if (!m_hash.empty())
-      {
-        md5 = CUtil::GetFileMD5(package);
-        if (!md5.Equals(m_hash))
-        {
-          CFile::Delete(package);
-          ReportInstallError(m_addon->ID(), URIUtils::GetFileName(package));
-          CLog::Log(LOGERROR, "MD5 mismatch after download %s", package.c_str());
-          return false;
-        }
-        db.AddPackage(m_addon->ID(), package, md5);
-      }
-
-      // check the archive as well - should have just a single folder in the root
-      CStdString archive;
-      URIUtils::CreateArchivePath(archive,"zip",package,"");
-
-      CFileItemList archivedFiles;
-      CDirectory::GetDirectory(archive, archivedFiles);
-
-      if (archivedFiles.Size() != 1 || !archivedFiles[0]->m_bIsFolder)
-      { // invalid package
-        db.RemovePackage(package);
-        CFile::Delete(package);
-        return false;
-      }
-      installFrom = archivedFiles[0]->GetPath();
+      CURL url(path);
+      url.SetProtocolOptions(m_referer);
+      path = url.Get();
     }
-    repoPtr.reset();
+    if (!CFile::Exists(package) && !DownloadPackage(path, dest))
+    {
+      CFile::Delete(package);
+      return false;
+    }
+
+    // at this point we have the package - check that it is valid
+    if (!CFile::Exists(package) || !CheckHash(package))
+    {
+      CFile::Delete(package);
+      return false;
+    }
+
+    // check the archive as well - should have just a single folder in the root
+    CStdString archive;
+    URIUtils::CreateArchivePath(archive,"zip",package,"");
+
+    CFileItemList archivedFiles;
+    CDirectory::GetDirectory(archive, archivedFiles);
+
+    if (archivedFiles.Size() != 1 || !archivedFiles[0]->m_bIsFolder)
+    { // invalid package
+      CFile::Delete(package);
+      return false;
+    }
+    installFrom = archivedFiles[0]->GetPath();
   }
 
   // run any pre-install functions
   bool reloadAddon = OnPreInstall();
 
   // perform install
-  if (!Install(installFrom, repoPtr))
+  if (!Install(installFrom))
     return false; // something went wrong
 
   // run any post-install guff
@@ -620,7 +548,7 @@ bool CAddonInstallJob::DownloadPackage(const CStdString &path, const CStdString
 bool CAddonInstallJob::OnPreInstall()
 {
   // check whether this is an active skin - we need to unload it if so
-  if (CSettings::Get().GetString("lookandfeel.skin") == m_addon->ID())
+  if (g_guiSettings.GetString("lookandfeel.skin") == m_addon->ID())
   {
     CApplicationMessenger::Get().ExecBuiltIn("UnloadSkin", true);
     return true;
@@ -628,8 +556,10 @@ bool CAddonInstallJob::OnPreInstall()
 
   if (m_addon->Type() == ADDON_SERVICE)
   {
-    bool running = !CAddonMgr::Get().IsAddonDisabled(m_addon->ID()); //grab a current state
-    CAddonMgr::Get().DisableAddon(m_addon->ID(),false); // enable it so we can remove it??
+    CAddonDatabase database;
+    database.Open();
+    bool running = !database.IsAddonDisabled(m_addon->ID()); //grab a current state
+    database.DisableAddon(m_addon->ID(),false); // enable it so we can remove it??
     // regrab from manager to have the correct path set
     AddonPtr addon;
     ADDON::CAddonMgr::Get().GetAddon(m_addon->ID(), addon);
@@ -657,11 +587,33 @@ bool CAddonInstallJob::DeleteAddon(const CStdString &addonFolder)
   return job.DoWork();
 }
 
-bool CAddonInstallJob::Install(const CStdString &installFrom, const AddonPtr& repo)
+bool CAddonInstallJob::Install(const CStdString &installFrom)
 {
-  // The first thing we do is install dependencies
-  ADDONDEPS deps = m_addon->GetDeps();
-  CStdString referer = StringUtils::Format("Referer=%s-%s.zip",m_addon->ID().c_str(),m_addon->Version().asString().c_str());
+  CStdString addonFolder(installFrom);
+  URIUtils::RemoveSlashAtEnd(addonFolder);
+  addonFolder = URIUtils::AddFileToFolder("special://home/addons/",
+                                       URIUtils::GetFileName(addonFolder));
+
+  CFileItemList install;
+  install.Add(CFileItemPtr(new CFileItem(installFrom, true)));
+  install[0]->Select(true);
+  CFileOperationJob job(CFileOperationJob::ActionReplace, install, "special://home/addons/");
+
+  AddonPtr addon;
+  if (!job.DoWork() || !CAddonMgr::Get().LoadAddonDescription(addonFolder, addon))
+  { // failed extraction or failed to load addon description
+    CStdString addonID = URIUtils::GetFileName(addonFolder);
+    ReportInstallError(addonID, addonID);
+    CLog::Log(LOGERROR,"Could not read addon description of %s", addonID.c_str());
+    DeleteAddon(addonFolder);
+    return false;
+  }
+
+  // resolve dependencies
+  CAddonMgr::Get().FindAddons(); // needed as GetDeps() grabs directly from c-pluff via the addon manager
+  ADDONDEPS deps = addon->GetDeps();
+  CStdString referer;
+  referer.Format("Referer=%s-%s.zip",addon->ID().c_str(),addon->Version().c_str());
   for (ADDONDEPS::iterator it  = deps.begin(); it != deps.end(); ++it)
   {
     if (it->first.Equals("xbmc.metadata"))
@@ -686,52 +638,18 @@ bool CAddonInstallJob::Install(const CStdString &installFrom, const AddonPtr& re
       }
       // don't have the addon or the addon isn't new enough - grab it (no new job for these)
       if (!CAddonInstaller::Get().Install(addonID, force, referer, false))
+      {
+        DeleteAddon(addonFolder);
         return false;
+      }
     }
   }
-
-  // now that we have all our dependencies, we can install our add-on
-  if (repo)
-  {
-    CFileItemList dummy;
-    CStdString s = StringUtils::Format("plugin://%s/?action=install"
-                                       "&package=%s&version=%s", repo->ID().c_str(),
-                                       m_addon->ID().c_str(),
-                                       m_addon->Version().asString().c_str());
-    if (!CDirectory::GetDirectory(s, dummy))
-      return false;
-  }
-  else
-  {
-    CStdString addonFolder(installFrom);
-    URIUtils::RemoveSlashAtEnd(addonFolder);
-    addonFolder = URIUtils::AddFileToFolder("special://home/addons/",
-                                         URIUtils::GetFileName(addonFolder));
-
-    CFileItemList install;
-    install.Add(CFileItemPtr(new CFileItem(installFrom, true)));
-    install[0]->Select(true);
-    CFileOperationJob job(CFileOperationJob::ActionReplace, install, "special://home/addons/");
-
-    AddonPtr addon;
-    if (!job.DoWork() || !CAddonMgr::Get().LoadAddonDescription(addonFolder, addon))
-    { // failed extraction or failed to load addon description
-      CStdString addonID = URIUtils::GetFileName(addonFolder);
-      ReportInstallError(addonID, addonID);
-      CLog::Log(LOGERROR,"Could not read addon description of %s", addonID.c_str());
-      DeleteAddon(addonFolder);
-      return false;
-    }
-
-    // Update the addon manager so that it has the newly installed add-on.
-    CAddonMgr::Get().FindAddons();
-  }
   return true;
 }
 
 void CAddonInstallJob::OnPostInstall(bool reloadAddon)
 {
-  if (CSettings::Get().GetBool("general.addonnotifications"))
+  if (m_addon->Type() < ADDON_VIZ_LIBRARY && g_settings.m_bAddonNotifications)
   {
     CGUIDialogKaiToast::QueueNotification(m_addon->Icon(),
                                           m_addon->Name(),
@@ -744,22 +662,22 @@ void CAddonInstallJob::OnPostInstall(bool reloadAddon)
     if (reloadAddon || (!m_update && CGUIDialogYesNo::ShowAndGetInput(m_addon->Name(),
                                                         g_localizeStrings.Get(24099),"","")))
     {
+      g_guiSettings.SetString("lookandfeel.skin",m_addon->ID().c_str());
       CGUIDialogKaiToast *toast = (CGUIDialogKaiToast *)g_windowManager.GetWindow(WINDOW_DIALOG_KAI_TOAST);
       if (toast)
       {
         toast->ResetTimer();
         toast->Close(true);
       }
-      if (CSettings::Get().GetString("lookandfeel.skin") == m_addon->ID())
-        CApplicationMessenger::Get().ExecBuiltIn("ReloadSkin", true);
-      else
-        CSettings::Get().SetString("lookandfeel.skin",m_addon->ID().c_str());
+      CApplicationMessenger::Get().ExecBuiltIn("ReloadSkin");
     }
   }
 
   if (m_addon->Type() == ADDON_SERVICE)
   {
-    CAddonMgr::Get().DisableAddon(m_addon->ID(),!reloadAddon); //return it into state it was before OnPreInstall()
+    CAddonDatabase database;
+    database.Open();
+    database.DisableAddon(m_addon->ID(),!reloadAddon); //return it into state it was before OnPreInstall()
     if (reloadAddon) // reload/start it if it was running
     {
       // regrab from manager to have the correct path set
@@ -810,6 +728,21 @@ void CAddonInstallJob::ReportInstallError(const CStdString& addonID,
   }
 }
 
+bool CAddonInstallJob::CheckHash(const CStdString& zipFile)
+{
+  if (m_hash.IsEmpty())
+    return true;
+  CStdString md5 = CUtil::GetFileMD5(zipFile);
+  if (!md5.Equals(m_hash))
+  {
+    CFile::Delete(zipFile);
+    ReportInstallError(m_addon->ID(), URIUtils::GetFileName(zipFile));
+    CLog::Log(LOGERROR, "MD5 mismatch after download %s", zipFile.c_str());
+    return false;
+  }
+  return true;
+}
+
 CStdString CAddonInstallJob::AddonID() const
 {
   return (m_addon) ? m_addon->ID() : "";
@@ -833,22 +766,8 @@ bool CAddonUnInstallJob::DoWork()
     if (service)
       service->Stop();
   }
-
-  AddonPtr repoPtr = CAddonInstallJob::GetRepoForAddon(m_addon);
-  RepositoryPtr therepo = boost::dynamic_pointer_cast<CRepository>(repoPtr);
-  if (therepo && !therepo->Props().libname.empty())
-  {
-    CFileItemList dummy;
-    CStdString s = StringUtils::Format("plugin://%s/?action=uninstall"
-                                       "&package=%s", therepo->ID().c_str(), m_addon->ID().c_str());
-    if (!CDirectory::GetDirectory(s, dummy))
-      return false;
-  }
-  else
-  {
-    if (!CAddonInstallJob::DeleteAddon(m_addon->Path()))
-      return false;
-  }
+  if (!CAddonInstallJob::DeleteAddon(m_addon->Path()))
+    return false;
 
   OnPostUnInstall();
 
@@ -866,10 +785,10 @@ void CAddonUnInstallJob::OnPostUnInstall()
 
   bool bSave(false);
   CFileItemList items;
-  XFILE::CFavouritesDirectory::Load(items);
+  CFavourites::Load(items);
   for (int i=0; i < items.Size(); ++i)
   {
-    if (items[i]->GetPath().find(m_addon->ID()) != std::string::npos)
+    if (items[i]->GetPath().Find(m_addon->ID()) > -1)
     {
       items.Remove(items[i].get());
       bSave = true;
@@ -877,11 +796,11 @@ void CAddonUnInstallJob::OnPostUnInstall()
   }
 
   if (bSave)
-    CFavouritesDirectory::Save(items);
+    CFavourites::Save(items);
 
   if (m_addon->Type() == ADDON_PVRDLL)
   {
-    if (CSettings::Get().GetBool("pvrmanager.enabled"))
+    if (g_guiSettings.GetBool("pvrmanager.enabled"))
       PVR::CPVRManager::Get().Start(true);
   }
 }
