@@ -1,6 +1,6 @@
 /*
- *      Copyright (C) 2010-2013 Team XBMC
- *      http://xbmc.org
+ *      Copyright (C) 2010-2012 Team XBMC
+ *      http://www.xbmc.org
  *
  *  This Program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,18 +26,19 @@
 #include <sstream>
 
 #include "AESinkALSA.h"
-#include "cores/AudioEngine/Utils/AEUtil.h"
-#include "cores/AudioEngine/Utils/AEELDParser.h"
+#include "Utils/AEUtil.h"
+#include "Utils/AEELDParser.h"
 #include "utils/StdString.h"
 #include "utils/log.h"
 #include "utils/MathUtils.h"
 #include "threads/SingleLock.h"
-#include "settings/AdvancedSettings.h"
-#if defined(HAS_LIBAMCODEC)
-#include "utils/AMLUtils.h"
+#include "settings/GUISettings.h"
+#if defined(HAS_AMLPLAYER)
+#include "cores/amlplayer/AMLUtils.h"
 #endif
 
-#define ALSA_OPTIONS (SND_PCM_NO_AUTO_FORMAT | SND_PCM_NO_AUTO_CHANNELS | SND_PCM_NO_AUTO_RESAMPLE)
+#define ALSA_OPTIONS (SND_PCM_NONBLOCK | SND_PCM_NO_AUTO_FORMAT | SND_PCM_NO_AUTO_CHANNELS | SND_PCM_NO_AUTO_RESAMPLE)
+#define ALSA_PERIODS 16
 
 #define ALSA_MAX_CHANNELS 16
 static enum AEChannel ALSAChannelMap[ALSA_MAX_CHANNELS + 1] = {
@@ -46,18 +47,6 @@ static enum AEChannel ALSAChannelMap[ALSA_MAX_CHANNELS + 1] = {
   AE_CH_NULL
 };
 
-static enum AEChannel ALSAChannelMap51Wide[ALSA_MAX_CHANNELS + 1] = {
-  AE_CH_FL      , AE_CH_FR      , AE_CH_SL      , AE_CH_SR      , AE_CH_FC      , AE_CH_LFE     , AE_CH_BL      , AE_CH_BR      ,
-  AE_CH_UNKNOWN1, AE_CH_UNKNOWN2, AE_CH_UNKNOWN3, AE_CH_UNKNOWN4, AE_CH_UNKNOWN5, AE_CH_UNKNOWN6, AE_CH_UNKNOWN7, AE_CH_UNKNOWN8, /* for p16v devices */
-  AE_CH_NULL
-};
-
-static enum AEChannel ALSAChannelMapPassthrough[ALSA_MAX_CHANNELS + 1] = {
-  AE_CH_RAW     , AE_CH_RAW     , AE_CH_RAW     , AE_CH_RAW     , AE_CH_RAW     , AE_CH_RAW     , AE_CH_RAW      , AE_CH_RAW      ,
-  AE_CH_UNKNOWN1, AE_CH_UNKNOWN2, AE_CH_UNKNOWN3, AE_CH_UNKNOWN4, AE_CH_UNKNOWN5, AE_CH_UNKNOWN6, AE_CH_UNKNOWN7, AE_CH_UNKNOWN8, /* for p16v devices */
-  AE_CH_NULL
-};
-
 static unsigned int ALSASampleRateList[] =
 {
   5512,
@@ -78,11 +67,7 @@ static unsigned int ALSASampleRateList[] =
 };
 
 CAESinkALSA::CAESinkALSA() :
-  m_bufferSize(0),
-  m_formatSampleRateMul(0.0),
-  m_passthrough(false),
-  m_pcm(NULL),
-  m_timeout(0)
+  m_pcm(NULL)
 {
   /* ensure that ALSA has been initialized */
   if (!snd_config)
@@ -94,55 +79,31 @@ CAESinkALSA::~CAESinkALSA()
   Deinitialize();
 }
 
-inline CAEChannelInfo CAESinkALSA::GetChannelLayout(AEAudioFormat format, unsigned int minChannels, unsigned int maxChannels)
+inline CAEChannelInfo CAESinkALSA::GetChannelLayout(AEAudioFormat format)
 {
-  enum AEChannel* channelMap = ALSAChannelMap;
   unsigned int count = 0;
 
-  if (format.m_dataFormat == AE_FMT_AC3 ||
-      format.m_dataFormat == AE_FMT_DTS ||
-      format.m_dataFormat == AE_FMT_EAC3)
-  {
-    count = 2;
-    channelMap = ALSAChannelMapPassthrough;
-  }
+       if (format.m_dataFormat == AE_FMT_AC3 ||
+           format.m_dataFormat == AE_FMT_DTS ||
+           format.m_dataFormat == AE_FMT_EAC3)
+           count = 2;
   else if (format.m_dataFormat == AE_FMT_TRUEHD ||
            format.m_dataFormat == AE_FMT_DTSHD)
-  {
-    count = 8;
-    channelMap = ALSAChannelMapPassthrough;
-  }
+           count = 8;
   else
   {
-    // According to CEA-861-D only RL and RR are known. In case of a format having SL and SR channels
-    // but no BR BL channels, we use the wide map in order to open only the num of channels really
-    // needed.
-    if (format.m_channelLayout.HasChannel(AE_CH_SL) && !format.m_channelLayout.HasChannel(AE_CH_BL))
-    {
-      channelMap = ALSAChannelMap51Wide;
-    }
     for (unsigned int c = 0; c < 8; ++c)
-    {
       for (unsigned int i = 0; i < format.m_channelLayout.Count(); ++i)
-      {
-        if (format.m_channelLayout[i] == channelMap[c])
+        if (format.m_channelLayout[i] == ALSAChannelMap[c])
         {
           count = c + 1;
           break;
         }
-      }
-    }
-    count = std::max(count, minChannels);
   }
 
   CAEChannelInfo info;
-  count = std::min(count, maxChannels);
   for (unsigned int i = 0; i < count; ++i)
-    info += channelMap[i];
-
-  CLog::Log(LOGDEBUG, "CAESinkALSA::GetChannelLayout - Input Channel Count: %d Output Channel Count: %d", format.m_channelLayout.Count(), count);
-  CLog::Log(LOGDEBUG, "CAESinkALSA::GetChannelLayout - Requested Layout: %s", std::string(format.m_channelLayout).c_str());
-  CLog::Log(LOGDEBUG, "CAESinkALSA::GetChannelLayout - Got Layout: %s", std::string(info).c_str());
+    info += ALSAChannelMap[i];
 
   return info;
 }
@@ -168,25 +129,22 @@ void CAESinkALSA::GetAESParams(AEAudioFormat format, std::string& params)
 
 bool CAESinkALSA::Initialize(AEAudioFormat &format, std::string &device)
 {
-  CAEChannelInfo channelLayout = GetChannelLayout(format, 2, 8);
   m_initDevice = device;
   m_initFormat = format;
-  ALSAConfig inconfig, outconfig;
-  inconfig.format = format.m_dataFormat;
-  inconfig.sampleRate = format.m_sampleRate;
-  inconfig.channels = channelLayout.Count();
 
   /* if we are raw, correct the data format */
   if (AE_IS_RAW(format.m_dataFormat))
   {
-    inconfig.format   = AE_FMT_S16NE;
-    m_passthrough     = true;
+    m_channelLayout     = GetChannelLayout(format);
+    format.m_dataFormat = AE_FMT_S16NE;
+    m_passthrough       = true;
   }
   else
   {
+    m_channelLayout = GetChannelLayout(format);
     m_passthrough   = false;
   }
-#if defined(HAS_LIBAMCODEC)
+#if defined(HAS_AMLPLAYER)
   if (aml_present())
   {
     aml_set_audio_passthrough(m_passthrough);
@@ -194,12 +152,14 @@ bool CAESinkALSA::Initialize(AEAudioFormat &format, std::string &device)
   }
 #endif
 
-  if (inconfig.channels == 0)
+  if (m_channelLayout.Count() == 0)
   {
     CLog::Log(LOGERROR, "CAESinkALSA::Initialize - Unable to open the requested channel layout");
     return false;
   }
 
+  format.m_channelLayout = m_channelLayout;
+
   AEDeviceType devType = AEDeviceTypeFromName(device);
 
   std::string AESParams;
@@ -214,7 +174,16 @@ bool CAESinkALSA::Initialize(AEAudioFormat &format, std::string &device)
   snd_config_t *config;
   snd_config_copy(&config, snd_config);
 
-  if (!OpenPCMDevice(device, AESParams, inconfig.channels, &m_pcm, config))
+  snd_config_t *dmixRateConf;
+  long dmixRate;
+
+  if (snd_config_search(config, "defaults.pcm.dmix.rate", &dmixRateConf) < 0
+      || snd_config_get_integer(dmixRateConf, &dmixRate) < 0)
+    dmixRate = 48000; /* assume default */
+
+
+  /* Prefer dmix for non-passthrough stereo when sample rate matches */
+  if (!OpenPCMDevice(device, AESParams, m_channelLayout.Count(), &m_pcm, config, format.m_sampleRate == (unsigned int) dmixRate && !m_passthrough))
   {
     CLog::Log(LOGERROR, "CAESinkALSA::Initialize - failed to initialize device \"%s\"", device.c_str());
     snd_config_delete(config);
@@ -230,32 +199,29 @@ bool CAESinkALSA::Initialize(AEAudioFormat &format, std::string &device)
   /* free the sound config */
   snd_config_delete(config);
 
-  if (!InitializeHW(inconfig, outconfig) || !InitializeSW(outconfig))
+  if (!InitializeHW(format) || !InitializeSW(format))
     return false;
 
-  // we want it blocking
-  snd_pcm_nonblock(m_pcm, 0);
+  snd_pcm_nonblock(m_pcm, 1);
   snd_pcm_prepare (m_pcm);
 
-  if (m_passthrough && inconfig.channels != outconfig.channels)
-  {
-    CLog::Log(LOGINFO, "CAESinkALSA::Initialize - could not open required number of channels");
-    return false;
-  }
-  // adjust format to the configuration we got
-  format.m_channelLayout = GetChannelLayout(format, outconfig.channels, outconfig.channels);
-  format.m_sampleRate = outconfig.sampleRate;
-  format.m_frames = outconfig.periodSize;
-  format.m_frameSize = outconfig.frameSize;
-  format.m_frameSamples = outconfig.periodSize * outconfig.channels;
-  format.m_dataFormat = outconfig.format;
-
   m_format              = format;
   m_formatSampleRateMul = 1.0 / (double)m_format.m_sampleRate;
 
   return true;
 }
 
+bool CAESinkALSA::IsCompatible(const AEAudioFormat format, const std::string device)
+{
+  return (
+      /* compare against the requested format and the real format */
+      (m_initFormat.m_sampleRate    == format.m_sampleRate    || m_format.m_sampleRate    == format.m_sampleRate   ) &&
+      (m_initFormat.m_dataFormat    == format.m_dataFormat    || m_format.m_dataFormat    == format.m_dataFormat   ) &&
+      (m_initFormat.m_channelLayout == format.m_channelLayout || m_format.m_channelLayout == format.m_channelLayout) &&
+      (m_initDevice == device)
+  );
+}
+
 snd_pcm_format_t CAESinkALSA::AEFormatToALSAFormat(const enum AEDataFormat format)
 {
   if (AE_IS_RAW(format))
@@ -263,6 +229,7 @@ snd_pcm_format_t CAESinkALSA::AEFormatToALSAFormat(const enum AEDataFormat forma
 
   switch (format)
   {
+    case AE_FMT_S8    : return SND_PCM_FORMAT_S8;
     case AE_FMT_U8    : return SND_PCM_FORMAT_U8;
     case AE_FMT_S16NE : return SND_PCM_FORMAT_S16;
     case AE_FMT_S16LE : return SND_PCM_FORMAT_S16_LE;
@@ -281,7 +248,7 @@ snd_pcm_format_t CAESinkALSA::AEFormatToALSAFormat(const enum AEDataFormat forma
   }
 }
 
-bool CAESinkALSA::InitializeHW(const ALSAConfig &inconfig, ALSAConfig &outconfig)
+bool CAESinkALSA::InitializeHW(AEAudioFormat &format)
 {
   snd_pcm_hw_params_t *hw_params;
 
@@ -291,35 +258,35 @@ bool CAESinkALSA::InitializeHW(const ALSAConfig &inconfig, ALSAConfig &outconfig
   snd_pcm_hw_params_any(m_pcm, hw_params);
   snd_pcm_hw_params_set_access(m_pcm, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED);
 
-  unsigned int sampleRate   = inconfig.sampleRate;
-  unsigned int channelCount = inconfig.channels;
+  unsigned int sampleRate   = format.m_sampleRate;
+  unsigned int channelCount = format.m_channelLayout.Count();
   snd_pcm_hw_params_set_rate_near    (m_pcm, hw_params, &sampleRate, NULL);
   snd_pcm_hw_params_set_channels_near(m_pcm, hw_params, &channelCount);
 
   /* ensure we opened X channels or more */
-  if (inconfig.channels > channelCount)
+  if (format.m_channelLayout.Count() > channelCount)
   {
     CLog::Log(LOGINFO, "CAESinkALSA::InitializeHW - Unable to open the required number of channels");
   }
 
-  /* update outconfig */
-  outconfig.channels = channelCount;
-
-  snd_pcm_format_t fmt = AEFormatToALSAFormat(inconfig.format);
-  outconfig.format = inconfig.format;
+  /* update the channelLayout to what we managed to open */
+  format.m_channelLayout.Reset();
+  for (unsigned int i = 0; i < channelCount; ++i)
+    format.m_channelLayout += ALSAChannelMap[i];
 
+  snd_pcm_format_t fmt = AEFormatToALSAFormat(format.m_dataFormat);
   if (fmt == SND_PCM_FORMAT_UNKNOWN)
   {
     /* if we dont support the requested format, fallback to float */
-    fmt = SND_PCM_FORMAT_FLOAT;
-    outconfig.format = AE_FMT_FLOAT;
+    format.m_dataFormat = AE_FMT_FLOAT;
+    fmt                 = SND_PCM_FORMAT_FLOAT;
   }
 
   /* try the data format */
   if (snd_pcm_hw_params_set_format(m_pcm, hw_params, fmt) < 0)
   {
     /* if the chosen format is not supported, try each one in decending order */
-    CLog::Log(LOGINFO, "CAESinkALSA::InitializeHW - Your hardware does not support %s, trying other formats", CAEUtil::DataFormatToStr(outconfig.format));
+    CLog::Log(LOGINFO, "CAESinkALSA::InitializeHW - Your hardware does not support %s, trying other formats", CAEUtil::DataFormatToStr(format.m_dataFormat));
     for (enum AEDataFormat i = AE_FMT_MAX; i > AE_FMT_INVALID; i = (enum AEDataFormat)((int)i - 1))
     {
       if (AE_IS_RAW(i) || i == AE_FMT_MAX)
@@ -348,8 +315,8 @@ bool CAESinkALSA::InitializeHW(const ALSAConfig &inconfig, ALSAConfig &outconfig
       }
 
       /* record that the format fell back to X */
-      outconfig.format = i;
-      CLog::Log(LOGINFO, "CAESinkALSA::InitializeHW - Using data format %s", CAEUtil::DataFormatToStr(outconfig.format));
+      format.m_dataFormat = i;
+      CLog::Log(LOGINFO, "CAESinkALSA::InitializeHW - Using data format %s", CAEUtil::DataFormatToStr(format.m_dataFormat));
       break;
     }
 
@@ -361,92 +328,65 @@ bool CAESinkALSA::InitializeHW(const ALSAConfig &inconfig, ALSAConfig &outconfig
     }
   }
 
+  unsigned int periods;
+
   snd_pcm_uframes_t periodSize, bufferSize;
   snd_pcm_hw_params_get_buffer_size_max(hw_params, &bufferSize);
-  snd_pcm_hw_params_get_period_size_max(hw_params, &periodSize, NULL);
 
-  /* 
-   We want to make sure, that we have max 200 ms Buffer with 
-   a periodSize of approx 50 ms. Choosing a higher bufferSize
-   will cause problems with menu sounds. Buffer will be increased
-   after those are fixed.
-  */
-  periodSize  = std::min(periodSize, (snd_pcm_uframes_t) sampleRate / 20);
-  bufferSize  = std::min(bufferSize, (snd_pcm_uframes_t) sampleRate / 5);
-  
-  /* 
-   According to upstream we should set buffer size first - so make sure it is always at least
-   4x period size to not get underruns (some systems seem to have issues with only 2 periods)
-  */
-  periodSize = std::min(periodSize, bufferSize / 4);
+  bufferSize  = std::min(bufferSize, (snd_pcm_uframes_t)8192);
+  periodSize  = bufferSize / ALSA_PERIODS;
+  periods     = ALSA_PERIODS;
 
-  CLog::Log(LOGDEBUG, "CAESinkALSA::InitializeHW - Request: periodSize %lu, bufferSize %lu", periodSize, bufferSize);
+  CLog::Log(LOGDEBUG, "CAESinkALSA::InitializeHW - Request: periodSize %lu, periods %u, bufferSize %lu", periodSize, periods, bufferSize);
 
+  /* work on a copy of the hw params */
   snd_pcm_hw_params_t *hw_params_copy;
   snd_pcm_hw_params_alloca(&hw_params_copy);
-  snd_pcm_hw_params_copy(hw_params_copy, hw_params); // copy what we have and is already working
 
-  // Make sure to not initialize too large to not cause underruns
-  snd_pcm_uframes_t periodSizeMax = bufferSize / 3;
-  if(snd_pcm_hw_params_set_period_size_max(m_pcm, hw_params_copy, &periodSizeMax, NULL) != 0)
+  /* try to set the buffer size then the period size */
+  snd_pcm_hw_params_copy(hw_params_copy, hw_params);
+  snd_pcm_hw_params_set_buffer_size_near(m_pcm, hw_params_copy, &bufferSize);
+  snd_pcm_hw_params_set_period_size_near(m_pcm, hw_params_copy, &periodSize, NULL);
+  snd_pcm_hw_params_set_periods_near    (m_pcm, hw_params_copy, &periods   , NULL);
+  if (snd_pcm_hw_params(m_pcm, hw_params_copy) != 0)
   {
-    snd_pcm_hw_params_copy(hw_params_copy, hw_params); // restore working copy
-    CLog::Log(LOGDEBUG, "CAESinkALSA::InitializeHW - Request: Failed to limit periodSize to %lu", periodSizeMax);
-  }
-  
-  // first trying bufferSize, PeriodSize
-  // for more info see here:
-  // http://mailman.alsa-project.org/pipermail/alsa-devel/2009-September/021069.html
-  // the last three tries are done as within pulseaudio
-
-  // backup periodSize and bufferSize first. Restore them after every failed try
-  snd_pcm_uframes_t periodSizeTemp, bufferSizeTemp;
-  periodSizeTemp = periodSize;
-  bufferSizeTemp = bufferSize;
-  if (snd_pcm_hw_params_set_buffer_size_near(m_pcm, hw_params_copy, &bufferSize) != 0
-    || snd_pcm_hw_params_set_period_size_near(m_pcm, hw_params_copy, &periodSize, NULL) != 0
-    || snd_pcm_hw_params(m_pcm, hw_params_copy) != 0)
-  {
-    bufferSize = bufferSizeTemp;
-    periodSize = periodSizeTemp;
-    // retry with PeriodSize, bufferSize
-    snd_pcm_hw_params_copy(hw_params_copy, hw_params); // restore working copy
-    if (snd_pcm_hw_params_set_period_size_near(m_pcm, hw_params_copy, &periodSize, NULL) != 0
-      || snd_pcm_hw_params_set_buffer_size_near(m_pcm, hw_params_copy, &bufferSize) != 0
-      || snd_pcm_hw_params(m_pcm, hw_params_copy) != 0)
+    /* try to set the period size then the buffer size */
+    snd_pcm_hw_params_copy(hw_params_copy, hw_params);
+    snd_pcm_hw_params_set_period_size_near(m_pcm, hw_params_copy, &periodSize, NULL);
+    snd_pcm_hw_params_set_buffer_size_near(m_pcm, hw_params_copy, &bufferSize);
+    snd_pcm_hw_params_set_periods_near    (m_pcm, hw_params_copy, &periods   , NULL);
+    if (snd_pcm_hw_params(m_pcm, hw_params_copy) != 0)
     {
-      // try only periodSize
-      periodSize = periodSizeTemp;
-      snd_pcm_hw_params_copy(hw_params_copy, hw_params); // restore working copy
-      if(snd_pcm_hw_params_set_period_size_near(m_pcm, hw_params_copy, &periodSize, NULL) != 0 
-        || snd_pcm_hw_params(m_pcm, hw_params_copy) != 0)
+      /* try to just set the buffer size */
+      snd_pcm_hw_params_copy(hw_params_copy, hw_params);
+      snd_pcm_hw_params_set_buffer_size_near(m_pcm, hw_params_copy, &bufferSize);
+      snd_pcm_hw_params_set_periods_near    (m_pcm, hw_params_copy, &periods   , NULL);
+      if (snd_pcm_hw_params(m_pcm, hw_params_copy) != 0)
       {
-        // try only BufferSize
-        bufferSize = bufferSizeTemp;
-        snd_pcm_hw_params_copy(hw_params_copy, hw_params); // restory working copy
-        if (snd_pcm_hw_params_set_buffer_size_near(m_pcm, hw_params_copy, &bufferSize) != 0
-          || snd_pcm_hw_params(m_pcm, hw_params_copy) != 0)
+        /* try to just set the period size */
+        snd_pcm_hw_params_copy(hw_params_copy, hw_params);
+        snd_pcm_hw_params_set_period_size_near(m_pcm, hw_params_copy, &periodSize, NULL);
+        snd_pcm_hw_params_set_periods_near    (m_pcm, hw_params_copy, &periods   , NULL);
+        if (snd_pcm_hw_params(m_pcm, hw_params_copy) != 0)
         {
-          // set default that Alsa would choose
-          CLog::Log(LOGWARNING, "CAESinkAlsa::IntializeHW - Using default alsa values - set failed");
-          if (snd_pcm_hw_params(m_pcm, hw_params) != 0)
-          {
-            CLog::Log(LOGDEBUG, "CAESinkALSA::InitializeHW - Could not init a valid sink");
-            return false;
-          }
+          CLog::Log(LOGERROR, "CAESinkALSA::InitializeHW - Failed to set the parameters");
+          return false;
         }
       }
-      // reread values when alsa default was kept
-      snd_pcm_get_params(m_pcm, &bufferSize, &periodSize);
     }
   }
+
+  snd_pcm_hw_params_get_period_size(hw_params_copy, &periodSize, NULL);
+  snd_pcm_hw_params_get_buffer_size(hw_params_copy, &bufferSize);
   
-  CLog::Log(LOGDEBUG, "CAESinkALSA::InitializeHW - Got: periodSize %lu, bufferSize %lu", periodSize, bufferSize);
+
+  CLog::Log(LOGDEBUG, "CAESinkALSA::InitializeHW - Got: periodSize %lu, periods %u, bufferSize %lu", periodSize, periods, bufferSize);
 
   /* set the format parameters */
-  outconfig.sampleRate   = sampleRate;
-  outconfig.periodSize   = periodSize;
-  outconfig.frameSize    = snd_pcm_frames_to_bytes(m_pcm, 1);
+  format.m_sampleRate   = sampleRate;
+  format.m_frames       = periodSize;
+  format.m_frameSamples = periodSize * format.m_channelLayout.Count();
+  format.m_frameSize    = snd_pcm_frames_to_bytes(m_pcm, 1);
 
   m_bufferSize = (unsigned int)bufferSize;
   m_timeout    = std::ceil((double)(bufferSize * 1000) / (double)sampleRate);
@@ -456,7 +396,7 @@ bool CAESinkALSA::InitializeHW(const ALSAConfig &inconfig, ALSAConfig &outconfig
   return true;
 }
 
-bool CAESinkALSA::InitializeSW(const ALSAConfig &inconfig)
+bool CAESinkALSA::InitializeSW(AEAudioFormat &format)
 {
   snd_pcm_sw_params_t *sw_params;
   snd_pcm_uframes_t boundary;
@@ -469,7 +409,7 @@ bool CAESinkALSA::InitializeSW(const ALSAConfig &inconfig)
   snd_pcm_sw_params_set_silence_threshold(m_pcm, sw_params, 0);
   snd_pcm_sw_params_get_boundary         (sw_params, &boundary);
   snd_pcm_sw_params_set_silence_size     (m_pcm, sw_params, boundary);
-  snd_pcm_sw_params_set_avail_min        (m_pcm, sw_params, inconfig.periodSize);
+  snd_pcm_sw_params_set_avail_min        (m_pcm, sw_params, format.m_frames);
 
   if (snd_pcm_sw_params(m_pcm, sw_params) < 0)
   {
@@ -482,9 +422,10 @@ bool CAESinkALSA::InitializeSW(const ALSAConfig &inconfig)
 
 void CAESinkALSA::Deinitialize()
 {
+  Stop();
+
   if (m_pcm)
   {
-    Stop();
     snd_pcm_close(m_pcm);
     m_pcm = NULL;
   }
@@ -515,34 +456,69 @@ double CAESinkALSA::GetDelay()
   return (double)frames * m_formatSampleRateMul;
 }
 
+double CAESinkALSA::GetCacheTime()
+{
+  if (!m_pcm)
+    return 0.0;
+
+  int space = snd_pcm_avail_update(m_pcm);
+  if (space == 0)
+  {
+    snd_pcm_state_t state = snd_pcm_state(m_pcm);
+    if (state < 0)
+    {
+      HandleError("snd_pcm_state", state);
+      space = m_bufferSize;
+    }
+
+    if (state != SND_PCM_STATE_RUNNING && state != SND_PCM_STATE_PREPARED)
+      space = m_bufferSize;
+  }
+
+  return (double)(m_bufferSize - space) * m_formatSampleRateMul;
+}
+
 double CAESinkALSA::GetCacheTotal()
 {
   return (double)m_bufferSize * m_formatSampleRateMul;
 }
 
-unsigned int CAESinkALSA::AddPackets(uint8_t **data, unsigned int frames, unsigned int offset)
+unsigned int CAESinkALSA::AddPackets(uint8_t *data, unsigned int frames, bool hasAudio)
 {
   if (!m_pcm)
   {
-    CLog::Log(LOGERROR, "CAESinkALSA - Tried to add packets without a sink");
-    return INT_MAX;
+    SoftResume();
+    if(!m_pcm)
+      return 0;
+
+    CLog::Log(LOGDEBUG, "CAESinkALSA - the grAEken is hunger, feed it (I am the downmost fallback - fix your code)");
   }
 
-  void *buffer = data[0]+offset*m_format.m_frameSize;
-  int ret = snd_pcm_writei(m_pcm, buffer, frames);
+  int ret;
+
+  ret = snd_pcm_avail(m_pcm);
+  if (ret < 0) 
+  {
+    HandleError("snd_pcm_avail", ret);
+    ret = 0;
+  }
+
+  if ((unsigned int)ret < frames)
+  {
+    ret = snd_pcm_wait(m_pcm, m_timeout);
+    if (ret < 0)
+      HandleError("snd_pcm_wait", ret);
+  }
+
+  ret = snd_pcm_writei(m_pcm, (void*)data, frames);
   if (ret < 0)
   {
-    CLog::Log(LOGERROR, "CAESinkALSA - snd_pcm_writei(%d) %s - trying to recover", ret, snd_strerror(ret));
-    ret = snd_pcm_recover(m_pcm, ret, 1);
-    if(ret < 0)
+    HandleError("snd_pcm_writei(1)", ret);
+    ret = snd_pcm_writei(m_pcm, (void*)data, frames);
+    if (ret < 0)
     {
-      HandleError("snd_pcm_writei(1)", ret);
-      ret = snd_pcm_writei(m_pcm, buffer, frames);
-      if (ret < 0)
-      {
-        HandleError("snd_pcm_writei(2)", ret);
-        ret = 0;
-      }
+      HandleError("snd_pcm_writei(2)", ret);
+      ret = 0;
     }
   }
 
@@ -586,8 +562,9 @@ void CAESinkALSA::Drain()
   if (!m_pcm)
     return;
 
+  snd_pcm_nonblock(m_pcm, 0);
   snd_pcm_drain(m_pcm);
-  snd_pcm_prepare(m_pcm);
+  snd_pcm_nonblock(m_pcm, 1);
 }
 
 void CAESinkALSA::AppendParams(std::string &device, const std::string &params)
@@ -637,7 +614,7 @@ bool CAESinkALSA::TryDeviceWithParams(const std::string &name, const std::string
   return TryDevice(name, pcmp, lconf);
 }
 
-bool CAESinkALSA::OpenPCMDevice(const std::string &name, const std::string &params, int channels, snd_pcm_t **pcmp, snd_config_t *lconf)
+bool CAESinkALSA::OpenPCMDevice(const std::string &name, const std::string &params, int channels, snd_pcm_t **pcmp, snd_config_t *lconf, bool preferDmixStereo)
 {
  /* Special name denoting surroundXX mangling. This is needed for some
    * devices for multichannel to work. */
@@ -665,8 +642,12 @@ bool CAESinkALSA::OpenPCMDevice(const std::string &name, const std::string &para
           return true;
     }
 
-    /* Try "sysdefault" and "default" (they provide dmix if needed, and route
-     * audio to all extra channels on subdeviced cards),
+    /* If preferDmix is false, try non-dmix configuration first.
+     * This allows output with non-48000 sample rate if device is free */
+    if (!preferDmixStereo && TryDeviceWithParams("front" + openName, params, pcmp, lconf))
+      return true;
+
+    /* Try "sysdefault" and "default" (they provide dmix),
      * unless the selected devices is not DEV=0 of the card, in which case
      * "sysdefault" and "default" would point to another device.
      * "sysdefault" is a newish device name that won't be overwritten in case
@@ -685,8 +666,8 @@ bool CAESinkALSA::OpenPCMDevice(const std::string &name, const std::string &para
         return true;
     }
 
-    /* Try "front" (no dmix, no audio in other channels on subdeviced cards) */
-    if (TryDeviceWithParams("front" + openName, params, pcmp, lconf))
+    /* Try non-dmix "front" */
+    if (preferDmixStereo && TryDeviceWithParams("front" + openName, params, pcmp, lconf))
       return true;
 
   }
@@ -905,7 +886,7 @@ std::string CAESinkALSA::GetParamFromName(const std::string &name, const std::st
 void CAESinkALSA::EnumerateDevice(AEDeviceInfoList &list, const std::string &device, const std::string &description, snd_config_t *config)
 {
   snd_pcm_t *pcmhandle = NULL;
-  if (!OpenPCMDevice(device, "", ALSA_MAX_CHANNELS, &pcmhandle, config))
+  if (!OpenPCMDevice(device, "", ALSA_MAX_CHANNELS, &pcmhandle, config, false))
     return;
 
   snd_pcm_info_t *pcminfo;
@@ -979,21 +960,6 @@ void CAESinkALSA::EnumerateDevice(AEDeviceInfoList &list, const std::string &dev
             /* snd_hctl_close also closes ctlhandle */
             snd_hctl_close(hctl);
 
-            // regarding data formats we don't trust ELD
-            // push all passthrough formats to the list
-            AEDataFormatList::iterator it;
-            for (enum AEDataFormat i = AE_FMT_MAX; i > AE_FMT_INVALID; i = (enum AEDataFormat)((int)i - 1))
-            {
-              if (!AE_IS_RAW(i))
-                continue;
-              it = find(info.m_dataFormats.begin(), info.m_dataFormats.end(), i);
-              if (it == info.m_dataFormats.end())
-              {
-                info.m_dataFormats.push_back(i);
-                CLog::Log(LOGNOTICE, "CAESinkALSA::%s data format \"%s\" on device \"%s\" seems to be not supported.", __FUNCTION__, CAEUtil::DataFormatToStr(i), device.c_str());
-              }
-            }
-
             if (badHDMI)
             {
               /* 
@@ -1018,9 +984,6 @@ void CAESinkALSA::EnumerateDevice(AEDeviceInfoList &list, const std::string &dev
       if (!info.m_displayNameExtra.empty())
         info.m_displayNameExtra += ' ';
       info.m_displayNameExtra += "S/PDIF";
-
-      info.m_dataFormats.push_back(AE_FMT_AC3);
-      info.m_dataFormats.push_back(AE_FMT_DTS);
     }
     else if (info.m_displayNameExtra.empty())
     {
@@ -1073,7 +1036,7 @@ void CAESinkALSA::EnumerateDevice(AEDeviceInfoList &list, const std::string &dev
   {
     /* Reopen the device if needed on the special "surroundXX" cases */
     if (info.m_deviceType == AE_DEVTYPE_PCM && (i == 8 || i == 6 || i == 4))
-      OpenPCMDevice(device, "", i, &pcmhandle, config);
+      OpenPCMDevice(device, "", i, &pcmhandle, config, false);
 
     if (snd_pcm_hw_params_test_channels(pcmhandle, hwparams, i) >= 0)
     {
@@ -1172,11 +1135,30 @@ bool CAESinkALSA::GetELD(snd_hctl_t *hctl, int device, CAEDeviceInfo& info, bool
   return true;
 }
 
+bool CAESinkALSA::SoftSuspend()
+{
+  if(m_pcm) // it is still there
+   Deinitialize();
+
+  return true;
+}
+bool CAESinkALSA::SoftResume()
+{
+    // reinit all the clibber
+    bool ret = true; // all fine
+    if(!m_pcm)
+    {
+      if (!snd_config)
+        snd_config_update();
+
+      ret = Initialize(m_initFormat, m_initDevice);
+    }
+   //we want that AE loves us again - reinit when initialize failed
+   return ret; // force reinit if false
+}
+
 void CAESinkALSA::sndLibErrorHandler(const char *file, int line, const char *function, int err, const char *fmt, ...)
 {
-  if(!g_advancedSettings.CanLogComponent(LOGAUDIO))
-    return;
-
   va_list arg;
   va_start(arg, fmt);
   char *errorStr;
