@@ -1,6 +1,6 @@
 /*
- *      Copyright (C) 2005-2013 Team XBMC
- *      http://xbmc.org
+ *      Copyright (C) 2005-2012 Team XBMC
+ *      http://www.xbmc.org
  *
  *  This Program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@
  *
  */
 
-#if (defined HAVE_CONFIG_H) && (!defined TARGET_WINDOWS)
+#if (defined HAVE_CONFIG_H) && (!defined WIN32)
   #include "config.h"
-#elif defined(TARGET_WINDOWS)
+#elif defined(_WIN32)
 #include "system.h"
 #endif
 
@@ -32,17 +32,16 @@
 
 #include "linux/XMemUtils.h"
 #include "utils/BitstreamStats.h"
+#include "settings/GUISettings.h"
+#include "settings/Settings.h"
 
 #include "DVDDemuxers/DVDDemuxUtils.h"
-#include "cores/AudioEngine/Utils/AEUtil.h"
 #include "utils/MathUtils.h"
 #include "settings/AdvancedSettings.h"
 #include "settings/Settings.h"
 #include "utils/TimeUtils.h"
 
 #include "OMXPlayer.h"
-#include "linux/RBP.h"
-#include "cores/AudioEngine/AEFactory.h"
 
 #include <iostream>
 #include <sstream>
@@ -64,7 +63,7 @@ public:
 };
 
 OMXPlayerAudio::OMXPlayerAudio(OMXClock *av_clock, CDVDMessageQueue& parent)
-: CThread("OMXPlayerAudio")
+: CThread("COMXPlayerAudio")
 , m_messageQueue("audio")
 , m_messageParent(parent)
 {
@@ -73,32 +72,34 @@ OMXPlayerAudio::OMXPlayerAudio(OMXClock *av_clock, CDVDMessageQueue& parent)
   m_speed         = DVD_PLAYSPEED_NORMAL;
   m_started       = false;
   m_stalled       = false;
-  m_audioClock    = DVD_NOPTS_VALUE;
+  m_audioClock    = 0;
   m_buffer_empty  = false;
   m_nChannels     = 0;
   m_DecoderOpen   = false;
+  m_freq          = CurrentHostFrequency();
+  m_send_eos      = false;
   m_bad_state     = false;
   m_hints_current.Clear();
 
-  bool small_mem = g_RBP.GetArmMem() < 256;
-  m_messageQueue.SetMaxDataSize((small_mem ? 3:6) * 1024 * 1024);
+  m_av_clock->SetMasterClock(false);
 
+  m_messageQueue.SetMaxDataSize(3 * 1024 * 1024);
   m_messageQueue.SetMaxTimeSize(8.0);
-  m_passthrough = false;
-  m_use_hw_decode = false;
-  m_hw_decode = false;
-  m_silence = false;
-  m_flush = false;  
 }
 
 
 OMXPlayerAudio::~OMXPlayerAudio()
 {
   CloseStream(false);
+
+  m_DllBcmHost.Unload();
 }
 
 bool OMXPlayerAudio::OpenStream(CDVDStreamInfo &hints)
 {
+  if(!m_DllBcmHost.Load())
+    return false;
+
   m_bad_state = false;
 
   COMXAudioCodecOMX *codec = new COMXAudioCodecOMX();
@@ -134,17 +135,25 @@ void OMXPlayerAudio::OpenStream(CDVDStreamInfo &hints, COMXAudioCodecOMX *codec)
     m_hints.bitspersample = 16;
 
   m_speed           = DVD_PLAYSPEED_NORMAL;
-  m_audioClock      = DVD_NOPTS_VALUE;
+  m_audioClock      = 0;
+  m_error           = 0;
+  m_errorbuff       = 0;
+  m_errorcount      = 0;
+  m_integral        = 0;
+  m_skipdupcount    = 0;
+  m_prevskipped     = false;
+  m_syncclock       = true;
   m_hw_decode       = false;
+  m_errortime       = CurrentHostCounter();
   m_silence         = false;
   m_started         = false;
   m_flush           = false;
   m_nChannels       = 0;
+  m_synctype        = SYNC_DISCON;
   m_stalled         = m_messageQueue.GetPacketCount(CDVDMsg::DEMUXER_PACKET) == 0;
+  m_use_passthrough = (g_guiSettings.GetInt("audiooutput.mode") == AUDIO_HDMI) ? true : false ;
   m_use_hw_decode   = g_advancedSettings.m_omxHWAudioDecode;
-  m_format.m_dataFormat    = GetDataFormat(m_hints);
-  m_format.m_sampleRate    = 0;
-  m_format.m_channelLayout = 0;
+  m_send_eos        = false;
 }
 
 bool OMXPlayerAudio::CloseStream(bool bWaitForBuffers)
@@ -183,6 +192,110 @@ void OMXPlayerAudio::OnExit()
   CLog::Log(LOGNOTICE, "thread end: OMXPlayerAudio::OnExit()");
 }
 
+
+
+void OMXPlayerAudio::HandleSyncError(double duration)
+{
+  double clock = m_av_clock->GetClock();
+  double error = m_audioClock - clock;
+  int64_t now;
+
+  if( fabs(error) > DVD_MSEC_TO_TIME(100) || m_syncclock )
+  {
+    m_av_clock->Discontinuity(clock+error);
+    /*
+    if(m_speed == DVD_PLAYSPEED_NORMAL)
+    CLog::Log(LOGDEBUG, "OMXPlayerAudio:: Discontinuity - was:%f, should be:%f, error:%f\n", clock, clock+error, error);
+    */
+
+    m_errorbuff = 0;
+    m_errorcount = 0;
+    m_skipdupcount = 0;
+    m_error = 0;
+    m_syncclock = false;
+    m_errortime = m_av_clock->CurrentHostCounter();
+
+    return;
+  }
+
+  if (m_speed != DVD_PLAYSPEED_NORMAL)
+  {
+    m_errorbuff = 0;
+    m_errorcount = 0;
+    m_integral = 0;
+    m_skipdupcount = 0;
+    m_error = 0;
+    m_errortime = m_av_clock->CurrentHostCounter();
+    return;
+  }
+
+  //check if measured error for 1 second
+  now = m_av_clock->CurrentHostCounter();
+  if ((now - m_errortime) >= m_freq)
+  {
+    m_errortime = now;
+    m_error = m_errorbuff / m_errorcount;
+
+    m_errorbuff = 0;
+    m_errorcount = 0;
+
+    if (m_synctype == SYNC_DISCON)
+    {
+      double limit, error;
+
+      if (m_av_clock->GetRefreshRate(&limit) > 0)
+      {
+        //when the videoreferenceclock is running, the discontinuity limit is one vblank period
+        limit *= DVD_TIME_BASE;
+
+        //make error a multiple of limit, rounded towards zero,
+        //so it won't interfere with the sync methods in CXBMCRenderManager::WaitPresentTime
+        if (m_error > 0.0)
+          error = limit * floor(m_error / limit);
+        else
+          error = limit * ceil(m_error / limit);
+      }
+      else
+      {
+        limit = DVD_MSEC_TO_TIME(10);
+        error = m_error;
+      }
+
+      /*
+      limit = DVD_MSEC_TO_TIME(10);
+      error = m_error;
+      */
+
+      if (fabs(error) > limit - 0.001)
+      {
+        m_av_clock->Discontinuity(clock+error);
+        /*
+        if(m_speed == DVD_PLAYSPEED_NORMAL)
+          CLog::Log(LOGDEBUG, "COMXPlayerAudio:: Discontinuity - was:%f, should be:%f, error:%f", clock, clock+error, error);
+        */
+      }
+    }
+    /*
+    else if (m_synctype == SYNC_SKIPDUP && m_skipdupcount == 0 && fabs(m_error) > DVD_MSEC_TO_TIME(10))
+    if (m_skipdupcount == 0 && fabs(m_error) > DVD_MSEC_TO_TIME(10))
+    {
+      //check how many packets to skip/duplicate
+      m_skipdupcount = (int)(m_error / duration);
+      //if less than one frame off, see if it's more than two thirds of a frame, so we can get better in sync
+      if (m_skipdupcount == 0 && fabs(m_error) > duration / 3 * 2)
+        m_skipdupcount = (int)(m_error / (duration / 3 * 2));
+
+      if (m_skipdupcount > 0)
+        CLog::Log(LOGDEBUG, "OMXPlayerAudio:: Duplicating %i packet(s) of %.2f ms duration",
+                  m_skipdupcount, duration / DVD_TIME_BASE * 1000.0);
+      else if (m_skipdupcount < 0)
+        CLog::Log(LOGDEBUG, "OMXPlayerAudio:: Skipping %i packet(s) of %.2f ms duration ",
+                  m_skipdupcount * -1,  duration / DVD_TIME_BASE * 1000.0);
+    }
+    */
+  }
+}
+
 bool OMXPlayerAudio::CodecChange()
 {
   unsigned int old_bitrate = m_hints.bitrate;
@@ -194,18 +307,16 @@ bool OMXPlayerAudio::CodecChange()
     m_hints.samplerate = m_pAudioCodec->GetSampleRate();
   }
 
-  /* only check bitrate changes on AV_CODEC_ID_DTS, AV_CODEC_ID_AC3, AV_CODEC_ID_EAC3 */
-  if(m_hints.codec != AV_CODEC_ID_DTS && m_hints.codec != AV_CODEC_ID_AC3 && m_hints.codec != AV_CODEC_ID_EAC3)
+  /* only check bitrate changes on CODEC_ID_DTS, CODEC_ID_AC3, CODEC_ID_EAC3 */
+  if(m_hints.codec != CODEC_ID_DTS && m_hints.codec != CODEC_ID_AC3 && m_hints.codec != CODEC_ID_EAC3)
     new_bitrate = old_bitrate = 0;
-
-  // for passthrough we only care about the codec and the samplerate
-  bool minor_change = m_hints_current.channels       != m_hints.channels ||
-                      m_hints_current.bitspersample  != m_hints.bitspersample ||
-                      old_bitrate                    != new_bitrate;
-
+    
   if(m_hints_current.codec          != m_hints.codec ||
+     m_hints_current.channels       != m_hints.channels ||
      m_hints_current.samplerate     != m_hints.samplerate ||
-     (!m_passthrough && minor_change) || !m_DecoderOpen)
+     m_hints_current.bitspersample  != m_hints.bitspersample ||
+     old_bitrate                    != new_bitrate ||
+     !m_DecoderOpen)
   {
     m_hints_current = m_hints;
     return true;
@@ -225,12 +336,11 @@ bool OMXPlayerAudio::Decode(DemuxPacket *pkt, bool bDropPacket)
   const uint8_t *data_dec = pkt->pData;
   int            data_len = pkt->iSize;
 
-  if(!OMX_IS_RAW(m_format.m_dataFormat) && !bDropPacket)
+  if(!OMX_IS_RAW(m_format.m_dataFormat))
   {
-    double dts = pkt->dts, pts=pkt->pts;
     while(!m_bStop && data_len > 0)
     {
-      int len = m_pAudioCodec->Decode((BYTE *)data_dec, data_len, dts, pts);
+      int len = m_pAudioCodec->Decode((BYTE *)data_dec, data_len);
       if( (len < 0) || (len >  data_len) )
       {
         m_pAudioCodec->Reset();
@@ -241,7 +351,7 @@ bool OMXPlayerAudio::Decode(DemuxPacket *pkt, bool bDropPacket)
       data_len -= len;
 
       uint8_t *decoded;
-      int decoded_size = m_pAudioCodec->GetData(&decoded, dts, pts);
+      int decoded_size = m_pAudioCodec->GetData(&decoded);
 
       if(decoded_size <=0)
         continue;
@@ -263,7 +373,7 @@ bool OMXPlayerAudio::Decode(DemuxPacket *pkt, bool bDropPacket)
         if(m_flush)
           break;
 
-        if(m_omxAudio.GetSpace() < (unsigned int)decoded_size)
+        if(m_omxAudio.GetSpace() < (unsigned int)pkt->iSize)
         {
           Sleep(10);
           continue;
@@ -275,7 +385,7 @@ bool OMXPlayerAudio::Decode(DemuxPacket *pkt, bool bDropPacket)
           if(m_silence)
             memset(decoded, 0x0, decoded_size);
 
-          ret = m_omxAudio.AddPackets(decoded, decoded_size, dts, pts, m_pAudioCodec->GetFrameSize());
+          ret = m_omxAudio.AddPackets(decoded, decoded_size, m_audioClock, m_audioClock);
 
           if(ret != decoded_size)
           {
@@ -283,12 +393,18 @@ bool OMXPlayerAudio::Decode(DemuxPacket *pkt, bool bDropPacket)
           }
         }
 
+        int n = (m_nChannels * m_hints.bitspersample * m_hints.samplerate)>>3;
+        if (n > 0)
+          m_audioClock += ((double)decoded_size * DVD_TIME_BASE) / n;
+
+        if(m_speed == DVD_PLAYSPEED_NORMAL)
+          HandleSyncError((((double)decoded_size * DVD_TIME_BASE) / n));
         break;
 
       }
     }
   }
-  else if(!bDropPacket)
+  else
   {
     if(CodecChange())
     {
@@ -313,9 +429,12 @@ bool OMXPlayerAudio::Decode(DemuxPacket *pkt, bool bDropPacket)
         if(m_silence)
           memset(pkt->pData, 0x0, pkt->iSize);
 
-        m_omxAudio.AddPackets(pkt->pData, pkt->iSize, m_audioClock, m_audioClock, 0);
+        m_omxAudio.AddPackets(pkt->pData, pkt->iSize, m_audioClock, m_audioClock);
       }
 
+      if(m_speed == DVD_PLAYSPEED_NORMAL)
+        HandleSyncError(0);
+
       m_audioStats.AddSampleBytes(pkt->iSize);
 
       break;
@@ -325,6 +444,16 @@ bool OMXPlayerAudio::Decode(DemuxPacket *pkt, bool bDropPacket)
   if(bDropPacket)
     m_stalled = false;
 
+  if(m_omxAudio.GetCacheTime() < 0.1 /*&& min(99,m_messageQueue.GetLevel() + MathUtils::round_int(100.0/8.0*GetCacheTime())) > 10*/)
+  {
+    m_stalled = true;
+    if(!m_av_clock->OMXAudioBuffer() && m_av_clock->HasVideo() && m_speed == DVD_PLAYSPEED_NORMAL)
+    {
+      clock_gettime(CLOCK_REALTIME, &m_starttime);
+      m_av_clock->OMXAudioBufferStart();
+    }
+  }
+
   // signal to our parent that we have initialized
   if(m_started == false)
   {
@@ -364,19 +493,25 @@ void OMXPlayerAudio::Process()
       DemuxPacket* pPacket = ((CDVDMsgDemuxerPacket*)pMsg)->GetPacket();
       bool bPacketDrop     = ((CDVDMsgDemuxerPacket*)pMsg)->GetPacketDrop();
 
-      #ifdef _DEBUG
-      CLog::Log(LOGINFO, "Audio: dts:%.0f pts:%.0f size:%d (s:%d f:%d d:%d l:%d) s:%d %d/%d late:%d,%d", pPacket->dts, pPacket->pts,
-           (int)pPacket->iSize, m_started, m_flush, bPacketDrop, m_stalled, m_speed, 0, 0, (int)m_omxAudio.GetAudioRenderingLatency(), (int)m_hints_current.samplerate);
-      #endif
       if(Decode(pPacket, m_speed > DVD_PLAYSPEED_NORMAL || m_speed < 0 || bPacketDrop))
       {
-        // we are not running until something is cached in output device
-        if(m_stalled && m_omxAudio.GetCacheTime() > 0.0)
+        if (m_stalled && (m_omxAudio.GetCacheTime() > (AUDIO_BUFFER_SECONDS * 0.75f)))
         {
           CLog::Log(LOGINFO, "COMXPlayerAudio - Switching to normal playback");
           m_stalled = false;
+          if(m_av_clock->HasVideo() && m_av_clock->OMXAudioBuffer())
+            m_av_clock->OMXAudioBufferStop();
         }
       }
+      // hard unlock audio out buffering
+      clock_gettime(CLOCK_REALTIME, &m_endtime);
+      //int iLevel = min(99,m_messageQueue.GetLevel() + MathUtils::round_int(100.0/8.0*GetCacheTime()));
+      if(/*iLevel < 10 &&*/ m_stalled && (m_endtime.tv_sec - m_starttime.tv_sec) > 1)
+      {
+        m_stalled = false;
+        if(m_av_clock->HasVideo() && m_av_clock->OMXAudioBuffer())
+          m_av_clock->OMXAudioBufferStop();
+      }
     }
     else if (pMsg->IsType(CDVDMsg::GENERAL_SYNCHRONIZE))
     {
@@ -389,73 +524,78 @@ void OMXPlayerAudio::Process()
     { //player asked us to set internal clock
       CDVDMsgGeneralResync* pMsgGeneralResync = (CDVDMsgGeneralResync*)pMsg;
 
-      if (pMsgGeneralResync->m_clock && pMsgGeneralResync->m_timestamp != DVD_NOPTS_VALUE)
+      if (pMsgGeneralResync->m_timestamp != DVD_NOPTS_VALUE)
+        m_audioClock = pMsgGeneralResync->m_timestamp;
+
+      if (pMsgGeneralResync->m_clock)
       {
-        CLog::Log(LOGDEBUG, "CDVDPlayerAudio - CDVDMsg::GENERAL_RESYNC(%f, %f, 1)", m_audioClock, pMsgGeneralResync->m_timestamp);
-        m_av_clock->Discontinuity(pMsgGeneralResync->m_timestamp);
+        CLog::Log(LOGDEBUG, "COMXPlayerAudio - CDVDMsg::GENERAL_RESYNC(%f, 1)", m_audioClock);
+        m_av_clock->Discontinuity(m_audioClock);
+        //m_av_clock->OMXUpdateClock(m_audioClock);
       }
       else
-        CLog::Log(LOGDEBUG, "CDVDPlayerAudio - CDVDMsg::GENERAL_RESYNC(%f, 0)", m_audioClock);
-
+        CLog::Log(LOGDEBUG, "COMXPlayerAudio - CDVDMsg::GENERAL_RESYNC(%f, 0)", m_audioClock);
       m_flush = false;
-      m_audioClock = DVD_NOPTS_VALUE;
     }
     else if (pMsg->IsType(CDVDMsg::GENERAL_RESET))
     {
       CLog::Log(LOGDEBUG, "COMXPlayerAudio - CDVDMsg::GENERAL_RESET");
       if (m_pAudioCodec)
         m_pAudioCodec->Reset();
+      m_av_clock->Lock();
+      m_av_clock->OMXStop(false);
       m_omxAudio.Flush();
+      m_av_clock->OMXReset(false);
+      m_av_clock->UnLock();
       m_started = false;
-      m_audioClock = DVD_NOPTS_VALUE;
     }
     else if (pMsg->IsType(CDVDMsg::GENERAL_FLUSH))
     {
       CLog::Log(LOGDEBUG, "COMXPlayerAudio - CDVDMsg::GENERAL_FLUSH");
+      m_av_clock->Lock();
+      m_av_clock->OMXStop(false);
       m_omxAudio.Flush();
+      m_av_clock->OMXReset(false);
+      m_av_clock->UnLock();
+      m_syncclock = true;
       m_stalled   = true;
       m_started   = false;
 
       if (m_pAudioCodec)
         m_pAudioCodec->Reset();
-      m_audioClock = DVD_NOPTS_VALUE;
     }
     else if (pMsg->IsType(CDVDMsg::PLAYER_STARTED))
     {
-      CLog::Log(LOGDEBUG, "COMXPlayerAudio - CDVDMsg::PLAYER_STARTED %d", m_started);
       if(m_started)
         m_messageParent.Put(new CDVDMsgInt(CDVDMsg::PLAYER_STARTED, DVDPLAYER_AUDIO));
     }
-    else if (pMsg->IsType(CDVDMsg::PLAYER_DISPLAYTIME))
-    {
-      COMXPlayer::SPlayerState& state = ((CDVDMsgType<COMXPlayer::SPlayerState>*)pMsg)->m_value;
-      double pts = m_audioClock;
-      double stamp = m_av_clock->OMXMediaTime();
-
-      if(state.time_src == COMXPlayer::ETIMESOURCE_CLOCK)
-        state.time      = stamp == 0.0 ? state.time : DVD_TIME_TO_MSEC(stamp + state.time_offset);
-      else
-        state.time      = stamp == 0.0 || pts == DVD_NOPTS_VALUE ? state.time : state.time + DVD_TIME_TO_MSEC(stamp - pts);
-      state.timestamp = m_av_clock->GetAbsoluteClock();
-      state.player    = DVDPLAYER_AUDIO;
-      m_messageParent.Put(pMsg->Acquire());
-    }
     else if (pMsg->IsType(CDVDMsg::GENERAL_EOF))
     {
       CLog::Log(LOGDEBUG, "COMXPlayerAudio - CDVDMsg::GENERAL_EOF");
-      SubmitEOS();
+      WaitCompletion();
     }
     else if (pMsg->IsType(CDVDMsg::GENERAL_DELAY))
     {
-      double timeout = static_cast<CDVDMsgDouble*>(pMsg)->m_value;
-      CLog::Log(LOGDEBUG, "COMXPlayerAudio - CDVDMsg::GENERAL_DELAY(%f)", timeout);
+      if (m_speed != DVD_PLAYSPEED_PAUSE)
+      {
+        double timeout = static_cast<CDVDMsgDouble*>(pMsg)->m_value;
+
+        CLog::Log(LOGDEBUG, "COMXPlayerAudio - CDVDMsg::GENERAL_DELAY(%f)", timeout);
+
+        timeout *= (double)DVD_PLAYSPEED_NORMAL / abs(m_speed);
+        timeout += m_av_clock->GetAbsoluteClock();
+
+        while(!m_bStop && m_av_clock->GetAbsoluteClock() < timeout)
+          Sleep(1);
+      }
     }
     else if (pMsg->IsType(CDVDMsg::PLAYER_SETSPEED))
     {
-      if (m_speed != static_cast<CDVDMsgInt*>(pMsg)->m_value)
+      CLog::Log(LOGDEBUG, "COMXPlayerAudio - CDVDMsg::PLAYER_SETSPEED");
+      m_speed = static_cast<CDVDMsgInt*>(pMsg)->m_value;
+      if (m_speed != DVD_PLAYSPEED_NORMAL)
       {
-        m_speed = static_cast<CDVDMsgInt*>(pMsg)->m_value;
-        CLog::Log(LOGDEBUG, "COMXPlayerAudio - CDVDMsg::PLAYER_SETSPEED %d", m_speed);
+        m_syncclock = true;
       }
     }
     else if (pMsg->IsType(CDVDMsg::AUDIO_SILENCE))
@@ -469,7 +609,6 @@ void OMXPlayerAudio::Process()
     else if (pMsg->IsType(CDVDMsg::GENERAL_STREAMCHANGE))
     {
       COMXMsgAudioCodecChange* msg(static_cast<COMXMsgAudioCodecChange*>(pMsg));
-      CLog::Log(LOGDEBUG, "COMXPlayerAudio - CDVDMsg::GENERAL_STREAMCHANGE");
       OpenStream(msg->m_hints, msg->m_codec);
       msg->m_codec = NULL;
     }
@@ -505,6 +644,14 @@ bool OMXPlayerAudio::Passthrough() const
 AEDataFormat OMXPlayerAudio::GetDataFormat(CDVDStreamInfo hints)
 {
   AEDataFormat dataFormat = AE_FMT_S16NE;
+  bool hdmi_passthrough_dts = false;
+  bool hdmi_passthrough_ac3 = false;
+
+  if (m_DllBcmHost.vc_tv_hdmi_audio_supported(EDID_AudioFormat_eAC3, 2, EDID_AudioSampleRate_e44KHz, EDID_AudioSampleSize_16bit ) == 0)
+    hdmi_passthrough_ac3 = true;
+  if (m_DllBcmHost.vc_tv_hdmi_audio_supported(EDID_AudioFormat_eDTS, 2, EDID_AudioSampleRate_e44KHz, EDID_AudioSampleSize_16bit ) == 0)
+    hdmi_passthrough_dts = true;
+  //printf("Audio support AC3=%d, DTS=%d\n", hdmi_passthrough_ac3, hdmi_passthrough_dts);
 
   m_passthrough = false;
   m_hw_decode   = false;
@@ -512,26 +659,29 @@ AEDataFormat OMXPlayerAudio::GetDataFormat(CDVDStreamInfo hints)
   /* check our audio capabilties */
 
   /* pathrought is overriding hw decode*/
-  if(hints.codec == AV_CODEC_ID_AC3 && CAEFactory::SupportsRaw(AE_FMT_AC3, hints.samplerate) && !CSettings::Get().GetBool("audiooutput.dualaudio"))
+  if(AUDIO_IS_BITSTREAM(g_guiSettings.GetInt("audiooutput.mode")) && m_use_passthrough)
   {
-    dataFormat = AE_FMT_AC3;
-    m_passthrough = true;
-  }
-  if(hints.codec == AV_CODEC_ID_DTS && CAEFactory::SupportsRaw(AE_FMT_DTS, hints.samplerate) && !CSettings::Get().GetBool("audiooutput.dualaudio"))
-  {
-    dataFormat = AE_FMT_DTS;
-    m_passthrough = true;
+    if(hints.codec == CODEC_ID_AC3 && g_guiSettings.GetBool("audiooutput.ac3passthrough") && hdmi_passthrough_ac3)
+    {
+      dataFormat = AE_FMT_AC3;
+      m_passthrough = true;
+    }
+    if(hints.codec == CODEC_ID_DTS && g_guiSettings.GetBool("audiooutput.dtspassthrough") && hdmi_passthrough_dts)
+    {
+      dataFormat = AE_FMT_DTS;
+      m_passthrough = true;
+    }
   }
 
   /* hw decode */
   if(m_use_hw_decode && !m_passthrough)
   {
-    if(hints.codec == AV_CODEC_ID_AC3 && COMXAudio::CanHWDecode(m_hints.codec))
+    if(hints.codec == CODEC_ID_AC3 && COMXAudio::CanHWDecode(m_hints.codec))
     {
       dataFormat = AE_FMT_AC3;
       m_hw_decode = true;
     }
-    if(hints.codec == AV_CODEC_ID_DTS && COMXAudio::CanHWDecode(m_hints.codec))
+    if(hints.codec == CODEC_ID_DTS && COMXAudio::CanHWDecode(m_hints.codec))
     {
       dataFormat = AE_FMT_DTS;
       m_hw_decode = true;
@@ -541,10 +691,10 @@ AEDataFormat OMXPlayerAudio::GetDataFormat(CDVDStreamInfo hints)
   /* software path */
   if(!m_passthrough && !m_hw_decode)
   {
-    if (m_pAudioCodec && m_pAudioCodec->GetBitsPerSample() == 16)
-      dataFormat = AE_FMT_S16NE;
-    else
-      dataFormat = AE_FMT_FLOAT;
+    /* 6 channel have to be mapped to 8 for PCM */
+    if(m_nChannels > 4)
+      m_nChannels = 8;
+    dataFormat = AE_FMT_S16NE;
   }
 
   return dataFormat;
@@ -556,25 +706,33 @@ bool OMXPlayerAudio::OpenDecoder()
   m_passthrough = false;
   m_hw_decode   = false;
 
+  bool bSendParent = false;
+
   if(m_DecoderOpen)
   {
+    WaitCompletion();
     m_omxAudio.Deinitialize();
     m_DecoderOpen = false;
+    bSendParent = true;
   }
 
   /* setup audi format for audio render */
   m_format.m_sampleRate    = m_hints.samplerate;
+  m_format.m_channelLayout = m_pAudioCodec->GetChannelMap(); 
   /* GetDataFormat is setting up evrything */
   m_format.m_dataFormat = GetDataFormat(m_hints);
 
-  CAEChannelInfo channelMap;
-  if (m_pAudioCodec && !m_passthrough)
-    channelMap = m_pAudioCodec->GetChannelMap();
-  else if (m_passthrough)
-    // we just want to get the channel count right to stop OMXAudio.cpp rejecting stream
-    // the actual layout is not used
-    channelMap = AE_CH_LAYOUT_5_1;
-  bool bAudioRenderOpen = m_omxAudio.Initialize(m_format, m_av_clock, m_hints, channelMap, m_passthrough, m_hw_decode);
+  std::string device = "";
+  
+  if(g_guiSettings.GetInt("audiooutput.mode") == AUDIO_HDMI)
+    device = "hdmi";
+  else
+    device = "local";
+
+  m_av_clock->Lock();
+  m_av_clock->OMXStop(false);
+
+  bool bAudioRenderOpen = m_omxAudio.Initialize(m_format, device, m_av_clock, m_hints, m_passthrough, m_hw_decode);
 
   m_codec_name = "";
   m_bad_state  = !bAudioRenderOpen;
@@ -590,14 +748,31 @@ bool OMXPlayerAudio::OpenDecoder()
       m_codec_name.c_str(), m_nChannels, m_hints.samplerate, m_hints.bitspersample);
   }
 
+  m_av_clock->OMXStateExecute(false);
+  m_av_clock->HasAudio(bAudioRenderOpen);
+  m_av_clock->OMXReset(false);
+  m_av_clock->UnLock();
+
   m_started = false;
 
+  // TODO : Send FLUSH to parent, only if we had a valid open codec. 
+  // this is just a workaround to get the omx video decoder happy again
+  // This situation happens, for example where we have in the stream an audio codec change
+  if(bSendParent)
+    m_messageParent.Put(new CDVDMsg(CDVDMsg::GENERAL_FLUSH));
+
   return bAudioRenderOpen;
 }
 
 void OMXPlayerAudio::CloseDecoder()
 {
+  m_av_clock->Lock();
+  m_av_clock->OMXStop(false);
   m_omxAudio.Deinitialize();
+  m_av_clock->HasAudio(false);
+  m_av_clock->OMXReset(false);
+  m_av_clock->UnLock();
+
   m_DecoderOpen = false;
 }
 
@@ -611,41 +786,26 @@ double OMXPlayerAudio::GetCacheTime()
   return m_omxAudio.GetCacheTime();
 }
 
-double OMXPlayerAudio::GetCacheTotal()
-{
-  return m_omxAudio.GetCacheTotal();
-}
-
-void OMXPlayerAudio::SubmitEOS()
-{
-  if(!m_bad_state)
-    m_omxAudio.SubmitEOS();
-}
-
-bool OMXPlayerAudio::IsEOS()
-{
-  return m_bad_state || m_omxAudio.IsEOS();
-}
-
 void OMXPlayerAudio::WaitCompletion()
 {
-  unsigned int nTimeOut = AUDIO_BUFFER_SECONDS * 1000;
-  while(nTimeOut)
-  {
-    if(IsEOS())
-    {
-      CLog::Log(LOGDEBUG, "%s::%s - got eos\n", CLASSNAME, __func__);
-      break;
-    }
+  if(!m_send_eos && !m_bad_state)
+    m_omxAudio.WaitCompletion();
+  m_send_eos = true;
+}
 
-    if(nTimeOut == 0)
-    {
-      CLog::Log(LOGERROR, "%s::%s - wait for eos timed out\n", CLASSNAME, __func__);
-      break;
-    }
-    Sleep(50);
-    nTimeOut -= 50;
-  }
+void OMXPlayerAudio::RegisterAudioCallback(IAudioCallback *pCallback)
+{
+  m_omxAudio.RegisterAudioCallback(pCallback);
+}
+
+void OMXPlayerAudio::UnRegisterAudioCallback()
+{
+  m_omxAudio.UnRegisterAudioCallback();
+}
+
+void OMXPlayerAudio::SetCurrentVolume(float fVolume)
+{
+  m_omxAudio.SetCurrentVolume(fVolume);
 }
 
 void OMXPlayerAudio::SetSpeed(int speed)
