@@ -1,6 +1,6 @@
 /*
- *      Copyright (C) 2005-2013 Team XBMC
- *      http://xbmc.org
+ *      Copyright (C) 2005-2012 Team XBMC
+ *      http://www.xbmc.org
  *
  *  This Program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -46,7 +46,6 @@ const char* ADDON_GUID_RE = "^(\\{){0,1}[0-9a-fA-F]{8}\\-[0-9a-fA-F]{4}\\-[0-9a-
 
 /* empty string for use in returns by ref */
 const CStdString StringUtils::EmptyString = "";
-const std::string StringUtils::Empty = "";
 CStdString StringUtils::m_lastUUID = "";
 
 string StringUtils::Format(const char *fmt, ...)
@@ -99,122 +98,27 @@ string StringUtils::FormatV(const char *fmt, va_list args)
     cstr = new_cstr;
   }
 
-  free(cstr);
   return "";
 }
 
-wstring StringUtils::Format(const wchar_t *fmt, ...)
-{
-  va_list args;
-  va_start(args, fmt);
-  wstring str = FormatV(fmt, args);
-  va_end(args);
-  
-  return str;
-}
-
-wstring StringUtils::FormatV(const wchar_t *fmt, va_list args)
-{
-  if (fmt == NULL)
-    return L"";
-  
-  int size = FORMAT_BLOCK_SIZE;
-  va_list argCopy;
-  
-  wchar_t *cstr = reinterpret_cast<wchar_t*>(malloc(sizeof(wchar_t) * size));
-  if (cstr == NULL)
-    return L"";
-  
-  while (1)
-  {
-    va_copy(argCopy, args);
-    
-    int nActual = vswprintf(cstr, size, fmt, argCopy);
-    va_end(argCopy);
-    
-    if (nActual > -1 && nActual < size) // We got a valid result
-    {
-      wstring str(cstr, nActual);
-      free(cstr);
-      return str;
-    }
-    if (nActual > -1)                   // Exactly what we will need (glibc 2.1)
-      size = nActual + 1;
-    else                                // Let's try to double the size (glibc 2.0)
-      size *= 2;
-    
-    wchar_t *new_cstr = reinterpret_cast<wchar_t*>(realloc(cstr, sizeof(wchar_t) * size));
-    if (new_cstr == NULL)
-    {
-      free(cstr);
-      return L"";
-    }
-    
-    cstr = new_cstr;
-  }
-  
-  return L"";
-}
-
 void StringUtils::ToUpper(string &str)
 {
   transform(str.begin(), str.end(), str.begin(), ::toupper);
 }
 
-void StringUtils::ToUpper(wstring &str)
-{
-  transform(str.begin(), str.end(), str.begin(), ::towupper);
-}
-
 void StringUtils::ToLower(string &str)
 {
   transform(str.begin(), str.end(), str.begin(), ::tolower);
 }
 
-void StringUtils::ToLower(wstring &str)
-{
-  transform(str.begin(), str.end(), str.begin(), ::towlower);
-}
-
 bool StringUtils::EqualsNoCase(const std::string &str1, const std::string &str2)
 {
-  return EqualsNoCase(str1.c_str(), str2.c_str());
-}
-
-bool StringUtils::EqualsNoCase(const std::string &str1, const char *s2)
-{
-  return EqualsNoCase(str1.c_str(), s2);
-}
-
-bool StringUtils::EqualsNoCase(const char *s1, const char *s2)
-{
-  char c2; // we need only one char outside the loop
-  do
-  {
-    const char c1 = *s1++; // const local variable should help compiler to optimize
-    c2 = *s2++;
-    if (c1 != c2 && ::tolower(c1) != ::tolower(c2)) // This includes the possibility that one of the characters is the null-terminator, which implies a string mismatch.
-      return false;
-  } while (c2 != '\0'); // At this point, we know c1 == c2, so there's no need to test them both.
-  return true;
-}
-
-int StringUtils::CompareNoCase(const std::string &str1, const std::string &str2)
-{
-  return CompareNoCase(str1.c_str(), str2.c_str());
-}
-
-int StringUtils::CompareNoCase(const char *s1, const char *s2)
-{
-  char c2; // we need only one char outside the loop
-  do
-  {
-    const char c1 = *s1++; // const local variable should help compiler to optimize
-    c2 = *s2++;
-    if (c1 != c2 && ::tolower(c1) != ::tolower(c2)) // This includes the possibility that one of the characters is the null-terminator, which implies a string mismatch.
-      return ::tolower(c1) - ::tolower(c2);
-  } while (c2 != '\0'); // At this point, we know c1 == c2, so there's no need to test them both.
-  return 0;
+  string tmp1 = str1;
+  string tmp2 = str2;
+  ToLower(tmp1);
+  ToLower(tmp2);
+  
+  return tmp1.compare(tmp2) == 0;
 }
 
 string StringUtils::Left(const string &str, size_t count)
@@ -248,76 +152,22 @@ std::string& StringUtils::Trim(std::string &str)
   return TrimRight(str);
 }
 
-std::string& StringUtils::Trim(std::string &str, const char* const chars)
-{
-  TrimLeft(str, chars);
-  return TrimRight(str, chars);
-}
-
-// hack to ensure that std::string::iterator will be dereferenced as _unsigned_ char
-// without this hack "TrimX" functions failed on Win32 with UTF-8 strings
-static int isspace_c(char c)
-{
-  return ::isspace((unsigned char)c);
-}
-
 std::string& StringUtils::TrimLeft(std::string &str)
 {
-  str.erase(str.begin(), ::find_if(str.begin(), str.end(), ::not1(::ptr_fun(isspace_c))));
-  return str;
-}
-
-std::string& StringUtils::TrimLeft(std::string &str, const char* const chars)
-{
-  size_t nidx = str.find_first_not_of(chars);
-  str.erase(0, nidx);
+  str.erase(str.begin(), ::find_if(str.begin(), str.end(), ::not1(::ptr_fun<int, int>(::isspace))));
   return str;
 }
 
 std::string& StringUtils::TrimRight(std::string &str)
 {
-  str.erase(::find_if(str.rbegin(), str.rend(), ::not1(::ptr_fun(isspace_c))).base(), str.end());
-  return str;
-}
-
-std::string& StringUtils::TrimRight(std::string &str, const char* const chars)
-{
-  size_t nidx = str.find_last_not_of(chars);
-  str.erase(str.npos == nidx ? 0 : ++nidx);
-  return str;
-}
-
-std::string& StringUtils::RemoveDuplicatedSpacesAndTabs(std::string& str)
-{
-  std::string::iterator it = str.begin();
-  bool onSpace = false;
-  while(it != str.end())
-  {
-    if (*it == '\t')
-      *it = ' ';
-
-    if (*it == ' ')
-    {
-      if (onSpace)
-      {
-        it = str.erase(it);
-        continue;
-      }
-      else
-        onSpace = true;
-    }
-    else
-      onSpace = false;
-
-    ++it;
-  }
+  str.erase(::find_if(str.rbegin(), str.rend(), ::not1(::ptr_fun<int, int>(::isspace))).base(), str.end());
   return str;
 }
 
 int StringUtils::Replace(string &str, char oldChar, char newChar)
 {
   int replacedChars = 0;
-  for (string::iterator it = str.begin(); it != str.end(); ++it)
+  for (string::iterator it = str.begin(); it != str.end(); it++)
   {
     if (*it == oldChar)
     {
@@ -331,9 +181,6 @@ int StringUtils::Replace(string &str, char oldChar, char newChar)
 
 int StringUtils::Replace(std::string &str, const std::string &oldStr, const std::string &newStr)
 {
-  if (oldStr.empty())
-    return 0;
-
   int replacedChars = 0;
   size_t index = 0;
   
@@ -347,152 +194,79 @@ int StringUtils::Replace(std::string &str, const std::string &oldStr, const std:
   return replacedChars;
 }
 
-int StringUtils::Replace(std::wstring &str, const std::wstring &oldStr, const std::wstring &newStr)
+bool StringUtils::StartsWith(const std::string &str, const std::string &str2, bool useCase /* = false */)
 {
-  if (oldStr.empty())
-    return 0;
+  std::string left = StringUtils::Left(str, str2.size());
+  
+  if (useCase)
+    return left.compare(str2) == 0;
 
-  int replacedChars = 0;
-  size_t index = 0;
-
-  while (index < str.size() && (index = str.find(oldStr, index)) != string::npos)
-  {
-    str.replace(index, oldStr.size(), newStr);
-    index += newStr.size();
-    replacedChars++;
-  }
-
-  return replacedChars;
+  return StringUtils::EqualsNoCase(left, str2);
 }
 
-bool StringUtils::StartsWith(const std::string &str1, const std::string &str2)
+bool StringUtils::EndsWith(const std::string &str, const std::string &str2, bool useCase /* = false */)
 {
-  return str1.compare(0, str2.size(), str2) == 0;
+  std::string right = StringUtils::Right(str, str2.size());
+  
+  if (useCase)
+    return right.compare(str2) == 0;
+
+  return StringUtils::EqualsNoCase(right, str2);
 }
 
-bool StringUtils::StartsWith(const std::string &str1, const char *s2)
+void StringUtils::JoinString(const CStdStringArray &strings, const CStdString& delimiter, CStdString& result)
 {
-  return StartsWith(str1.c_str(), s2);
+  result = "";
+  for(CStdStringArray::const_iterator it = strings.begin(); it != strings.end(); it++ )
+    result += (*it) + delimiter;
+
+  if(result != "")
+    result.Delete(result.size()-delimiter.size(), delimiter.size());
 }
 
-bool StringUtils::StartsWith(const char *s1, const char *s2)
+CStdString StringUtils::JoinString(const CStdStringArray &strings, const CStdString& delimiter)
 {
-  while (*s2 != '\0')
-  {
-    if (*s1 != *s2)
-      return false;
-    s1++;
-    s2++;
-  }
-  return true;
-}
-
-bool StringUtils::StartsWithNoCase(const std::string &str1, const std::string &str2)
-{
-  return StartsWithNoCase(str1.c_str(), str2.c_str());
-}
-
-bool StringUtils::StartsWithNoCase(const std::string &str1, const char *s2)
-{
-  return StartsWithNoCase(str1.c_str(), s2);
-}
-
-bool StringUtils::StartsWithNoCase(const char *s1, const char *s2)
-{
-  while (*s2 != '\0')
-  {
-    if (::tolower(*s1) != ::tolower(*s2))
-      return false;
-    s1++;
-    s2++;
-  }
-  return true;
-}
-
-bool StringUtils::EndsWith(const std::string &str1, const std::string &str2)
-{
-  if (str1.size() < str2.size())
-    return false;
-  return str1.compare(str1.size() - str2.size(), str2.size(), str2) == 0;
-}
-
-bool StringUtils::EndsWith(const std::string &str1, const char *s2)
-{
-  size_t len2 = strlen(s2);
-  if (str1.size() < len2)
-    return false;
-  return str1.compare(str1.size() - len2, len2, s2) == 0;
-}
-
-bool StringUtils::EndsWithNoCase(const std::string &str1, const std::string &str2)
-{
-  if (str1.size() < str2.size())
-    return false;
-  const char *s1 = str1.c_str() + str1.size() - str2.size();
-  const char *s2 = str2.c_str();
-  while (*s2 != '\0')
-  {
-    if (::tolower(*s1) != ::tolower(*s2))
-      return false;
-    s1++;
-    s2++;
-  }
-  return true;
-}
-
-bool StringUtils::EndsWithNoCase(const std::string &str1, const char *s2)
-{
-  size_t len2 = strlen(s2);
-  if (str1.size() < len2)
-    return false;
-  const char *s1 = str1.c_str() + str1.size() - len2;
-  while (*s2 != '\0')
-  {
-    if (::tolower(*s1) != ::tolower(*s2))
-      return false;
-    s1++;
-    s2++;
-  }
-  return true;
+  CStdString result;
+  JoinString(strings, delimiter, result);
+  return result;
 }
 
 CStdString StringUtils::Join(const vector<string> &strings, const CStdString& delimiter)
 {
-  CStdString result;
-  for(vector<string>::const_iterator it = strings.begin(); it != strings.end(); it++ )
-    result += (*it) + delimiter.c_str();
-  
-  if (!result.empty())
-    result.erase(result.size() - delimiter.size());
-  return result;
+  CStdStringArray strArray;
+  for (unsigned int index = 0; index < strings.size(); index++)
+    strArray.push_back(strings.at(index));
+
+  return JoinString(strArray, delimiter);
 }
 
-vector<string> StringUtils::Split(const std::string& input, const std::string& delimiter, unsigned int iMaxStrings /* = 0 */)
+// Splits the string input into pieces delimited by delimiter.
+// if 2 delimiters are in a row, it will include the empty string between them.
+// added MaxStrings parameter to restrict the number of returned substrings (like perl and python)
+int StringUtils::SplitString(const CStdString& input, const CStdString& delimiter, CStdStringArray &results, unsigned int iMaxStrings /* = 0 */)
 {
-  vector<string> results;
-  if (input.empty())
-    return results;
+  int iPos = -1;
+  int newPos = -1;
+  int sizeS2 = delimiter.GetLength();
+  int isize = input.GetLength();
 
-  size_t iPos = std::string::npos;
-  size_t newPos = std::string::npos;
-  size_t sizeS2 = delimiter.size();
-  size_t isize = input.size();
+  results.clear();
 
   vector<unsigned int> positions;
 
-  newPos = input.find(delimiter, 0);
+  newPos = input.Find (delimiter, 0);
 
-  if (newPos == std::string::npos)
+  if ( newPos < 0 )
   {
     results.push_back(input);
-    return results;
+    return 1;
   }
 
-  while (newPos != std::string::npos)
+  while ( newPos > iPos )
   {
     positions.push_back(newPos);
     iPos = newPos;
-    newPos = input.find(delimiter, iPos + sizeS2);
+    newPos = input.Find (delimiter, iPos + sizeS2);
   }
 
   // numFound is the number of delimiters which is one less
@@ -503,40 +277,60 @@ vector<string> StringUtils::Split(const std::string& input, const std::string& d
 
   for ( unsigned int i = 0; i <= numFound; i++ )
   {
-    string s;
+    CStdString s;
     if ( i == 0 )
     {
       if ( i == numFound )
         s = input;
       else
-        s = input.substr(i, positions[i]);
+        s = input.Mid( i, positions[i] );
     }
     else
     {
-      size_t offset = positions[i - 1] + sizeS2;
+      int offset = positions[i - 1] + sizeS2;
       if ( offset < isize )
       {
         if ( i == numFound )
-          s = input.substr(offset);
+          s = input.Mid(offset);
         else if ( i > 0 )
-          s = input.substr( positions[i - 1] + sizeS2,
+          s = input.Mid( positions[i - 1] + sizeS2,
                          positions[i] - positions[i - 1] - sizeS2 );
       }
     }
     results.push_back(s);
   }
-  return results;
+  // return the number of substrings
+  return results.size();
+}
+
+CStdStringArray StringUtils::SplitString(const CStdString& input, const CStdString& delimiter, unsigned int iMaxStrings /* = 0 */)
+{
+  CStdStringArray result;
+  SplitString(input, delimiter, result, iMaxStrings);
+  return result;
+}
+
+vector<string> StringUtils::Split(const CStdString& input, const CStdString& delimiter, unsigned int iMaxStrings /* = 0 */)
+{
+  CStdStringArray result;
+  SplitString(input, delimiter, result, iMaxStrings);
+
+  vector<string> strArray;
+  for (unsigned int index = 0; index < result.size(); index++)
+    strArray.push_back(result.at(index));
+
+  return strArray;
 }
 
 // returns the number of occurrences of strFind in strInput.
 int StringUtils::FindNumber(const CStdString& strInput, const CStdString &strFind)
 {
-  size_t pos = strInput.find(strFind, 0);
+  int pos = strInput.Find(strFind, 0);
   int numfound = 0;
-  while (pos != std::string::npos)
+  while (pos >= 0)
   {
     numfound++;
-    pos = strInput.find(strFind, pos + 1);
+    pos = strInput.Find(strFind, pos + 1);
   }
   return numfound;
 }
@@ -609,12 +403,13 @@ int64_t StringUtils::AlphaNumericCompare(const wchar_t *left, const wchar_t *rig
 
 int StringUtils::DateStringToYYYYMMDD(const CStdString &dateString)
 {
-  vector<string> days = StringUtils::Split(dateString, "-");
-  if (days.size() == 1)
+  CStdStringArray days;
+  int splitCount = StringUtils::SplitString(dateString, "-", days);
+  if (splitCount == 1)
     return atoi(days[0].c_str());
-  else if (days.size() == 2)
+  else if (splitCount == 2)
     return atoi(days[0].c_str())*100+atoi(days[1].c_str());
-  else if (days.size() == 3)
+  else if (splitCount == 3)
     return atoi(days[0].c_str())*10000+atoi(days[1].c_str())*100+atoi(days[2].c_str());
   else
     return -1;
@@ -623,20 +418,22 @@ int StringUtils::DateStringToYYYYMMDD(const CStdString &dateString)
 long StringUtils::TimeStringToSeconds(const CStdString &timeString)
 {
   CStdString strCopy(timeString);
-  StringUtils::Trim(strCopy);
-  if(StringUtils::EndsWithNoCase(strCopy, " min"))
+  strCopy.TrimLeft(" \n\r\t");
+  strCopy.TrimRight(" \n\r\t");
+  if(strCopy.Right(4).Equals(" min"))
   {
     // this is imdb format of "XXX min"
     return 60 * atoi(strCopy.c_str());
   }
   else
   {
-    vector<string> secs = StringUtils::Split(strCopy, ":");
+    CStdStringArray secs;
+    StringUtils::SplitString(strCopy, ":", secs);
     int timeInSecs = 0;
     for (unsigned int i = 0; i < 3 && i < secs.size(); i++)
     {
       timeInSecs *= 60;
-      timeInSecs += atoi(secs[i].c_str());
+      timeInSecs += atoi(secs[i]);
     }
     return timeInSecs;
   }
@@ -653,13 +450,13 @@ CStdString StringUtils::SecondsToTimeString(long lSeconds, TIME_FORMAT format)
     format = (hh >= 1) ? TIME_FORMAT_HH_MM_SS : TIME_FORMAT_MM_SS;
   CStdString strHMS;
   if (format & TIME_FORMAT_HH)
-    strHMS += StringUtils::Format("%02.2i", hh);
+    strHMS.AppendFormat("%02.2i", hh);
   else if (format & TIME_FORMAT_H)
-    strHMS += StringUtils::Format("%i", hh);
+    strHMS.AppendFormat("%i", hh);
   if (format & TIME_FORMAT_MM)
-    strHMS += StringUtils::Format(strHMS.empty() ? "%02.2i" : ":%02.2i", mm);
+    strHMS.AppendFormat(strHMS.IsEmpty() ? "%02.2i" : ":%02.2i", mm);
   if (format & TIME_FORMAT_SS)
-    strHMS += StringUtils::Format(strHMS.empty() ? "%02.2i" : ":%02.2i", ss);
+    strHMS.AppendFormat(strHMS.IsEmpty() ? "%02.2i" : ":%02.2i", ss);
   return strHMS;
 }
 
@@ -695,31 +492,12 @@ bool StringUtils::IsInteger(const CStdString& str)
   return i == str.size() && n > 0;
 }
 
-int StringUtils::asciidigitvalue(char chr)
-{
-  if (!isasciidigit(chr))
-    return -1;
-
-  return chr - '0';
-}
-
-int StringUtils::asciixdigitvalue(char chr)
-{
-  int v = asciidigitvalue(chr);
-  if (v >= 0)
-    return v;
-  if (chr >= 'a' && chr <= 'f')
-    return chr - 'a' + 10;
-  if (chr >= 'A' && chr <= 'F')
-    return chr - 'A' + 10;
-
-  return -1;
-}
-
-
 void StringUtils::RemoveCRLF(CStdString& strLine)
 {
-  StringUtils::TrimRight(strLine, "\n\r");
+  while ( strLine.size() && (strLine.Right(1) == "\n" || strLine.Right(1) == "\r") )
+  {
+    strLine = strLine.Left(std::max(0, (int)strLine.size() - 1));
+  }
 }
 
 CStdString StringUtils::SizeToString(int64_t size)
@@ -735,45 +513,15 @@ CStdString StringUtils::SizeToString(int64_t size)
   }
 
   if (!i)
-    strLabel = StringUtils::Format("%.0lf %cB ", s, prefixes[i]);
+    strLabel.Format("%.0lf %cB ", s, prefixes[i]);
   else if (s >= 100.0)
-    strLabel = StringUtils::Format("%.1lf %cB", s, prefixes[i]);
+    strLabel.Format("%.1lf %cB", s, prefixes[i]);
   else
-    strLabel = StringUtils::Format("%.2lf %cB", s, prefixes[i]);
+    strLabel.Format("%.2lf %cB", s, prefixes[i]);
 
   return strLabel;
 }
 
-// return -1 if not, else return the utf8 char length.
-int IsUTF8Letter(const unsigned char *str)
-{
-  // reference:
-  // unicode -> utf8 table: http://www.utf8-chartable.de/
-  // latin characters in unicode: http://en.wikipedia.org/wiki/Latin_characters_in_Unicode
-  unsigned char ch = str[0];
-  if (!ch)
-    return -1;
-  if ((ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z'))
-    return 1;
-  if (!(ch & 0x80))
-    return -1;
-  unsigned char ch2 = str[1];
-  if (!ch2)
-    return -1;
-  // check latin 1 letter table: http://en.wikipedia.org/wiki/C1_Controls_and_Latin-1_Supplement
-  if (ch == 0xC3 && ch2 >= 0x80 && ch2 <= 0xBF && ch2 != 0x97 && ch2 != 0xB7)
-    return 2;
-  // check latin extended A table: http://en.wikipedia.org/wiki/Latin_Extended-A
-  if (ch >= 0xC4 && ch <= 0xC7 && ch2 >= 0x80 && ch2 <= 0xBF)
-    return 2;
-  // check latin extended B table: http://en.wikipedia.org/wiki/Latin_Extended-B
-  // and International Phonetic Alphabet: http://en.wikipedia.org/wiki/IPA_Extensions_(Unicode_block)
-  if (((ch == 0xC8 || ch == 0xC9) && ch2 >= 0x80 && ch2 <= 0xBF)
-      || (ch == 0xCA && ch2 >= 0x80 && ch2 <= 0xAF))
-    return 2;
-  return -1;
-}
-
 size_t StringUtils::FindWords(const char *str, const char *wordLowerCase)
 {
   // NOTE: This assumes word is lowercase!
@@ -796,20 +544,8 @@ size_t StringUtils::FindWords(const char *str, const char *wordLowerCase)
     if (same && *w == 0)  // only the same if word has been exhausted
       return (const char *)s - str;
 
-    // otherwise, skip current word (composed by latin letters) or number
-    int l;
-    if (*s >= '0' && *s <= '9')
-    {
-      ++s;
-      while (*s >= '0' && *s <= '9') ++s;
-    }
-    else if ((l = IsUTF8Letter(s)) > 0)
-    {
-      s += l;
-      while ((l = IsUTF8Letter(s)) > 0) s += l;
-    }
-    else
-      ++s;
+    // otherwise, find a space and skip to the end of the whitespace
+    while (*s && *s != ' ') s++;
     while (*s && *s == ' ') s++;
 
     // and repeat until we're done
@@ -840,7 +576,7 @@ int StringUtils::FindEndBracket(const CStdString &str, char opener, char closer,
 void StringUtils::WordToDigits(CStdString &word)
 {
   static const char word_to_letter[] = "22233344455566677778889999";
-  StringUtils::ToLower(word);
+  word.ToLower();
   for (unsigned int i = 0; i < word.size(); ++i)
   { // NB: This assumes ascii, which probably needs extending at some  point.
     char letter = word[i];
@@ -914,13 +650,13 @@ double StringUtils::CompareFuzzy(const CStdString &left, const CStdString &right
   return (0.5 + fstrcmp(left.c_str(), right.c_str(), 0.0) * (left.length() + right.length())) / 2.0;
 }
 
-int StringUtils::FindBestMatch(const CStdString &str, const vector<string> &strings, double &matchscore)
+int StringUtils::FindBestMatch(const CStdString &str, const CStdStringArray &strings, double &matchscore)
 {
   int best = -1;
   matchscore = 0;
 
   int i = 0;
-  for (vector<string>::const_iterator it = strings.begin(); it != strings.end(); ++it, i++)
+  for (CStdStringArray::const_iterator it = strings.begin(); it != strings.end(); it++, i++)
   {
     int maxlength = max(str.length(), it->length());
     double score = StringUtils::CompareFuzzy(str, *it) / maxlength;
@@ -933,16 +669,6 @@ int StringUtils::FindBestMatch(const CStdString &str, const vector<string> &stri
   return best;
 }
 
-bool StringUtils::ContainsKeyword(const CStdString &str, const vector<string> &keywords)
-{
-  for (vector<string>::const_iterator it = keywords.begin(); it != keywords.end(); ++it)
-  {
-    if (str.find(*it) != str.npos)
-      return true;
-  }
-  return false;
-}
-
 size_t StringUtils::utf8_strlen(const char *s)
 {
   size_t length = 0;
@@ -965,29 +691,3 @@ std::string StringUtils::Paramify(const std::string &param)
   // add double quotes around the whole string
   return "\"" + result + "\"";
 }
-
-std::vector<std::string> StringUtils::Tokenize(const std::string &input, const std::string &delimiters)
-{
-  std::vector<std::string> tokens;
-  Tokenize(input, tokens, delimiters);
-  return tokens;
-}
-
-void StringUtils::Tokenize(const std::string& input, std::vector<std::string>& tokens, const std::string& delimiters)
-{
-  // Tokenize ripped from http://www.linuxselfhelp.com/HOWTO/C++Programming-HOWTO-7.html
-  // Skip delimiters at beginning.
-  string::size_type lastPos = input.find_first_not_of(delimiters, 0);
-  // Find first "non-delimiter".
-  string::size_type pos = input.find_first_of(delimiters, lastPos);
-
-  while (string::npos != pos || string::npos != lastPos)
-  {
-    // Found a token, add it to the vector.
-    tokens.push_back(input.substr(lastPos, pos - lastPos));
-    // Skip delimiters.  Note the "not_of"
-    lastPos = input.find_first_not_of(delimiters, pos);
-    // Find next "non-delimiter"
-    pos = input.find_first_of(delimiters, lastPos);
-  }
-}
